# -*- coding: utf-8 -*-
"""
Adobe DNG SDK Conversion Process
================================

Defines various objects implementing raw conversion based on *Adobe DNG SDK*
and *dcraw*:

-   :func:`colour_hdri.convert_raw_files_to_dng_files`
-   :func:`colour_hdri.convert_dng_files_to_intermediate_files`
-   :func:`colour_hdri.read_dng_files_exif_tags`
"""

from __future__ import division, unicode_literals

import logging
import numpy as np
import os
import platform
import re
import shlex
import subprocess  # nosec
from copy import deepcopy

from colour.utilities import CaseInsensitiveMapping, warning
from colour.utilities.documentation import (DocstringText,
                                            is_documentation_building)

from colour_hdri.utilities import (ExifTag, parse_exif_array,
                                   parse_exif_numeric, parse_exif_string,
                                   path_exists, read_exif_tags)

__author__ = 'Colour Developers'
__copyright__ = 'Copyright (C) 2015-2020 - Colour Developers'
__license__ = 'New BSD License - https://opensource.org/licenses/BSD-3-Clause'
__maintainer__ = 'Colour Developers'
__email__ = 'colour-developers@colour-science.org'
__status__ = 'Production'

__all__ = [
    'RAW_CONVERTER', 'RAW_CONVERSION_ARGUMENTS', 'RAW_D_CONVERSION_ARGUMENTS',
    'DNG_CONVERTER', 'DNG_CONVERSION_ARGUMENTS', 'DNG_EXIF_TAGS_BINDING',
    'convert_raw_files_to_dng_files',
    'convert_dng_files_to_intermediate_files', 'read_dng_files_exif_tags'
]

_IS_MACOS_PLATFORM = platform.system() == 'Darwin'
"""
Whether the current platform is *macOS*.
"""

_IS_WINDOWS_PLATFORM = platform.system() in ('Windows', 'Microsoft')
"""
Whether the current platform is *Windows*.
"""

RAW_CONVERTER = 'dcraw'
if is_documentation_building():  # pragma: no cover
    RAW_CONVERTER = DocstringText(RAW_CONVERTER)
    RAW_CONVERTER.__doc__ = """
Command line raw conversion application, usually Dave Coffin's *dcraw*.

RAW_CONVERTER : unicode
"""

RAW_CONVERSION_ARGUMENTS = '-t 0 -D -W -4 -T "{0}"'
if _IS_WINDOWS_PLATFORM:
    RAW_CONVERSION_ARGUMENTS = RAW_CONVERSION_ARGUMENTS.replace('"', '')
if is_documentation_building():  # pragma: no cover
    RAW_CONVERSION_ARGUMENTS = DocstringText(RAW_CONVERSION_ARGUMENTS)
    RAW_CONVERSION_ARGUMENTS.__doc__ = """
Arguments for the command line raw conversion application for non
demosaiced linear *tiff* file format output.

RAW_CONVERSION_ARGUMENTS : unicode
"""

RAW_D_CONVERSION_ARGUMENTS = '-t 0 -H 1 -r 1 1 1 1 -4 -q 3 -o 0 -T "{0}"'
if _IS_WINDOWS_PLATFORM:
    RAW_D_CONVERSION_ARGUMENTS = RAW_D_CONVERSION_ARGUMENTS.replace('"', '')
if is_documentation_building():  # pragma: no cover
    RAW_D_CONVERSION_ARGUMENTS = DocstringText(RAW_D_CONVERSION_ARGUMENTS)
    RAW_D_CONVERSION_ARGUMENTS.__doc__ = """
Arguments for the command line raw conversion application for demosaiced
linear *tiff* file format output.

RAW_D_CONVERSION_ARGUMENTS : unicode
"""

if _IS_MACOS_PLATFORM:
    DNG_CONVERTER = ('/Applications/Adobe DNG Converter.app/Contents/'
                     'MacOS/Adobe DNG Converter')
elif _IS_WINDOWS_PLATFORM:
    DNG_CONVERTER = 'Adobe DNG Converter'
else:
    warning('"Adobe DNG Converter" is not available on your platform!')
    DNG_CONVERTER = None

if DNG_CONVERTER is not None:
    if is_documentation_building():  # pragma: no cover
        DNG_CONVERTER = DocstringText(DNG_CONVERTER)
        DNG_CONVERTER.__doc__ = """
Command line *DNG* conversion application, usually *Adobe DNG Converter*.

DNG_CONVERTER : unicode
"""

DNG_CONVERSION_ARGUMENTS = '-cr7.1 -l -d "{0}" "{1}"'
if _IS_WINDOWS_PLATFORM:
    DNG_CONVERSION_ARGUMENTS = DNG_CONVERSION_ARGUMENTS.replace('"', '')
if is_documentation_building():  # pragma: no cover
    DNG_CONVERSION_ARGUMENTS = DocstringText(DNG_CONVERSION_ARGUMENTS)
    DNG_CONVERSION_ARGUMENTS.__doc__ = """
Arguments for the command line *dng* conversion application.

DNG_CONVERSION_ARGUMENTS : unicode
"""

DNG_EXIF_TAGS_BINDING = CaseInsensitiveMapping({
    'EXIF':
        CaseInsensitiveMapping({
            'Make': (parse_exif_string, None),
            'Camera Model Name': (parse_exif_string, None),
            'Camera Serial Number': (parse_exif_string, None),
            'Lens Model': (parse_exif_string, None),
            'DNG Lens Info': (parse_exif_string, None),
            'Focal Length': (parse_exif_numeric, None),
            'Exposure Time': (parse_exif_numeric, None),
            'F Number': (parse_exif_numeric, None),
            'ISO': (parse_exif_numeric, None),
            'CFA Pattern 2': (lambda x: parse_exif_array(x, np.int_), None),
            'CFA Plane Color': (lambda x: parse_exif_array(x, np.int_), None),
            'Black Level Repeat Dim': (lambda x: parse_exif_array(x, np.int_),
                                       None),
            'Black Level': (lambda x: parse_exif_array(x, np.int_), None),
            'White Level': (lambda x: parse_exif_array(x, np.int_), None),
            'Samples Per Pixel': (lambda x: parse_exif_numeric(x, np.int_),
                                  None),
            'Active Area': (lambda x: parse_exif_array(x, np.int_), None),
            'Orientation': (lambda x: parse_exif_numeric(x, np.int_), None),
            'Camera Calibration Sig': (parse_exif_string, None),
            'Profile Calibration Sig': (parse_exif_string, None),
            'Calibration Illuminant 1': (
                lambda x: parse_exif_numeric(x, np.int_), 17),
            'Calibration Illuminant 2': (
                lambda x: parse_exif_numeric(x, np.int_), 21),
            'Color Matrix 1': (
                lambda x: parse_exif_array(x, np.float_, (3, 3)),
                '1 0 0 0 1 0 0 0 1'),
            'Color Matrix 2': (
                lambda x: parse_exif_array(x, np.float_, (3, 3)),
                '1 0 0 0 1 0 0 0 1'),
            'Camera Calibration 1': (
                lambda x: parse_exif_array(x, np.float_, (3, 3)),
                '1 0 0 0 1 0 0 0 1'),
            'Camera Calibration 2': (
                lambda x: parse_exif_array(x, np.float_, (3, 3)),
                '1 0 0 0 1 0 0 0 1'),
            'Analog Balance': (lambda x: parse_exif_array(x, np.float_),
                               '1 1 1'),
            'Reduction Matrix 1': (
                lambda x: parse_exif_array(x, np.float_, (3, 3)),
                '1 0 0 0 1 0 0 0 1'),
            'Reduction Matrix 2': (
                lambda x: parse_exif_array(x, np.float_, (3, 3)),
                '1 0 0 0 1 0 0 0 1'),
            'Forward Matrix 1': (
                lambda x: parse_exif_array(x, np.float_, (3, 3)),
                '1 0 0 0 1 0 0 0 1'),
            'Forward Matrix 2': (
                lambda x: parse_exif_array(x, np.float_, (3, 3)),
                '1 0 0 0 1 0 0 0 1'),
            'As Shot Neutral': (lambda x: parse_exif_array(x, np.float_),
                                '1 1 1'),
            'Baseline Exposure': (lambda x: parse_exif_numeric(x, np.float_),
                                  None),
            'Baseline Noise': (lambda x: parse_exif_numeric(x, np.float_),
                               None)
        })
})
DNG_EXIF_TAGS_BINDING.__doc__ = """
Exif tags binding for a *dng* file.

DNG_EXIF_TAGS_BINDING : CaseInsensitiveMapping
"""


def convert_raw_files_to_dng_files(raw_files, output_directory):
    """
    Converts given raw files to *dng* files using given output directory.

    Parameters
    ----------
    raw_files : array_like
        Raw files to convert to *dng* files.
    output_directory : unicode
        Output directory.

    Returns
    -------
    list
        *dng* files.
    """

    dng_files = []
    for raw_file in raw_files:
        raw_file_extension = os.path.splitext(raw_file)[1]
        dng_file = os.path.join(
            output_directory,
            os.path.basename(
                re.sub('{0}$'.format(raw_file_extension), '.dng', raw_file)))

        if path_exists(dng_file):
            os.remove(dng_file)

        logging.info('Converting "{0}" file to "{1}" file.'.format(
            raw_file, dng_file))

        command = [DNG_CONVERTER] + shlex.split(
            DNG_CONVERSION_ARGUMENTS.format(output_directory, raw_file),
            posix=not _IS_WINDOWS_PLATFORM)

        subprocess.call(command, shell=_IS_WINDOWS_PLATFORM)  # nosec

        dng_files.append(dng_file)

    return dng_files


def convert_dng_files_to_intermediate_files(dng_files,
                                            output_directory,
                                            demosaicing=False):
    """
    Converts given *dng* files to intermediate *tiff* files using given output
    directory.

    Parameters
    ----------
    dng_files : array_like
        *dng* files to convert to intermediate *tiff* files.
    output_directory : str
        Output directory.
    demosaicing : bool
        Perform demosaicing on conversion.

    Returns
    -------
    list
        Intermediate *tiff* files.
    """

    intermediate_files = []
    for dng_file in dng_files:
        intermediate_file = re.sub('\\.dng$', '.tiff', dng_file)

        if path_exists(intermediate_file):
            os.remove(intermediate_file)

        logging.info('Converting "{0}" file to "{1}" file.'.format(
            dng_file, intermediate_file))

        raw_conversion_arguments = (RAW_D_CONVERSION_ARGUMENTS if demosaicing
                                    else RAW_CONVERSION_ARGUMENTS)
        command = [RAW_CONVERTER] + shlex.split(
            raw_conversion_arguments.format(dng_file),
            posix=not _IS_WINDOWS_PLATFORM)

        subprocess.call(command, shell=_IS_WINDOWS_PLATFORM)  # nosec

        tiff_file = os.path.join(output_directory,
                                 os.path.basename(intermediate_file))
        if tiff_file != intermediate_file:
            if path_exists(tiff_file):
                os.remove(tiff_file)

            os.rename(intermediate_file, tiff_file)

        intermediate_files.append(tiff_file)

    return intermediate_files


def read_dng_files_exif_tags(dng_files,
                             exif_tags_binding=DNG_EXIF_TAGS_BINDING):
    """
    Reads given *dng* files exif tags using given binding.

    Parameters
    ----------
    dng_files : array_like
        *dng* files to read the exif tags from.
    exif_tags_binding : dict_like
        Exif tags binding.

    Returns
    -------
    list
        *dng* files exif tags.
    """

    dng_files_exif_tags = []
    for dng_file in dng_files:
        exif_tags = read_exif_tags(dng_file)
        binding = deepcopy(exif_tags_binding)
        for group, tags in binding.items():
            for tag in tags:
                exif_tag = exif_tags[group].get(tag)
                parser = binding[group][tag][0]
                if exif_tag is None:
                    default = binding[group][tag][1]
                    binding[group][tag] = (
                        default if default is None else parser(
                            ExifTag(value=binding[group][tag][1])))
                else:
                    binding[group][tag] = parser(exif_tag[0])

        dng_files_exif_tags.append(binding)

    return dng_files_exif_tags



# -*- coding: utf-8 -*-
"""
Debevec (1997) Camera Response Function Computation
===================================================

Defines *Debevec (1997)* camera responses computation objects:

-   :func:`colour_hdri.g_solve`
-   :func:`colour_hdri.camera_response_functions_Debevec1997`

See Also
--------
`Colour - HDRI - Examples: Merge from Low Dynamic Range Files Jupyter Notebook
<https://github.com/colour-science/colour-hdri/\
blob/master/colour_hdri/examples/examples_merge_from_ldr_files.ipynb>`__

References
----------
-   :cite:`Debevec1997a` : Debevec, P. E., & Malik, J. (1997). Recovering high
    dynamic range radiance maps from photographs. Proceedings of the 24th
    Annual Conference on Computer Graphics and Interactive Techniques -
    SIGGRAPH "97, August, 369-378. doi:10.1145/258734.258884
"""

from __future__ import division, unicode_literals

import numpy as np

from colour.utilities import as_float_array, as_int_array, tstack

from colour_hdri.exposure import average_luminance
from colour_hdri.generation import weighting_function_Debevec1997
from colour_hdri.sampling import samples_Grossberg2003

__author__ = 'Colour Developers'
__copyright__ = 'Copyright (C) 2015-2020 - Colour Developers'
__license__ = 'New BSD License - https://opensource.org/licenses/BSD-3-Clause'
__maintainer__ = 'Colour Developers'
__email__ = 'colour-developers@colour-science.org'
__status__ = 'Production'

__all__ = ['g_solve', 'camera_response_functions_Debevec1997']


def g_solve(Z, B, l_s=30, w=weighting_function_Debevec1997, n=256):
    """
    Given a set of pixel values observed for several pixels in several images
    with different exposure times, this function returns the imaging system's
    response function :math:`g` as well as the log film irradiance values
    :math:`lE` for the observed pixels.

    Parameters
    ----------
    Z : array_like
        Set of pixel values observed for several pixels in several images.
    B : array_like
        Log :math:`\\Delta t`, or log shutter speed for images.
    l_s : numeric, optional
        :math:`\\lambda` smoothing term.
    w : callable, optional
        Weighting function :math:`w`.
    n : int, optional
        :math:`n` constant.

    Returns
    -------
    tuple
        Camera response functions :math:`g(z)` and log film irradiance values
        :math:`lE`.

    References
    ----------
    :cite:`Debevec1997a`
    """

    Z = as_int_array(Z)
    B = as_float_array(B)
    l_s = as_float_array(l_s)

    Z_x, Z_y = Z.shape

    A = np.zeros((Z_x * Z_y + n - 1, n + Z_x))
    b = np.zeros((A.shape[0], 1))
    w = w(np.linspace(0, 1, n))

    k = 0
    for i in np.arange(Z_x):
        for j in np.arange(Z_y):
            Z_c = Z[i, j]
            w_ij = w[Z_c]
            A[k, Z_c] = w_ij
            A[k, n + i] = -w_ij
            b[k] = w_ij * B[j]
            k += 1

    A[k, np.int_(n / 2)] = 1
    k += 1

    for i in np.arange(1, n - 1, 1):
        A[k, i - 1] = l_s * w[i]
        A[k, i + 0] = l_s * w[i] * -2
        A[k, i + 1] = l_s * w[i]
        k += 1

    x = np.squeeze(np.linalg.lstsq(A, b)[0])

    g = x[0:n]
    lE = x[n:x.shape[0]]

    return g, lE


def camera_response_functions_Debevec1997(image_stack,
                                          s=samples_Grossberg2003,
                                          samples=1000,
                                          l_s=30,
                                          w=weighting_function_Debevec1997,
                                          n=256,
                                          normalise=True):
    """
    Returns the camera response functions for given image stack using
    *Debevec (1997)* method.

    Image channels are sampled with :math:`s` sampling function and the output
    samples are passed to :func:`colour_hdri.g_solve`.

    Parameters
    ----------
    image_stack : colour_hdri.ImageStack
        Stack of single channel or multi-channel floating point images.
    s : callable, optional
        Sampling function :math:`s`.
    samples : int, optional
        Samples count per images.
    l_s : numeric, optional
        :math:`\\lambda` smoothing term.
    w : callable, optional
        Weighting function :math:`w`.
    n : int, optional
        :math:`n` constant.
    normalise : bool, optional
        Enables the camera response functions normalisation. Uncertain camera
        response functions values resulting from :math:`w` function are
        set to zero.

    Returns
    -------
    ndarray
        Camera response functions :math:`g(z)`.

    References
    ----------
    :cite:`Debevec1997a`
    """

    s_o = s(image_stack.data, samples, n)

    L_l = np.log(1 / average_luminance(
        image_stack.f_number, image_stack.exposure_time, image_stack.iso))

    g_c = [
        g_solve(s_o[..., x], L_l, l_s, w, n)[0] for x in range(s_o.shape[-1])
    ]
    crfs = np.exp(tstack(np.array(g_c)))

    if normalise:
        # TODO: Investigate if the normalisation value should account for the
        # percentage of uncertain camera response functions values or be
        # correlated to it and scaled according. As an alternative of setting
        # the uncertain camera response functions values to zero, it would be
        # interesting to explore extrapolation as the camera response functions
        # are essentially smooth. It is important to note that camera sensors
        # are usually acting non linearly when reaching saturation level.
        crfs[w(np.linspace(0, 1, crfs.shape[0])) == 0] = 0
        crfs /= np.max(crfs, axis=0)

    return crfs



# -*- coding: utf-8 -*-

from __future__ import absolute_import
from .common import vivification, vivified_to_dict, path_exists, filter_files
from .exif import (EXIF_EXECUTABLE, ExifTag, parse_exif_string,
                   parse_exif_numeric, parse_exif_fraction, parse_exif_array,
                   parse_exif_data, read_exif_tags, copy_exif_tags,
                   update_exif_tags, delete_exif_tags, read_exif_tag,
                   write_exif_tag)
from .image import Metadata, Image, ImageStack

__all__ = []
__all__ += ['vivification', 'vivified_to_dict', 'path_exists', 'filter_files']
__all__ += [
    'EXIF_EXECUTABLE', 'ExifTag', 'parse_exif_string', 'parse_exif_numeric',
    'parse_exif_fraction', 'parse_exif_array', 'parse_exif_data',
    'read_exif_tags', 'copy_exif_tags', 'update_exif_tags', 'delete_exif_tags',
    'read_exif_tag', 'write_exif_tag'
]
__all__ += ['Metadata', 'Image', 'ImageStack']



# -*- coding: utf-8 -*-
"""
HDRI / Radiance Image Plotting
==============================

Defines the HDRI / radiance image plotting objects:

-   :func:`colour_hdri.plotting.plot_radiance_image_strip`
"""

from __future__ import division, unicode_literals

import matplotlib
import matplotlib.pyplot as plt
import numpy as np

from colour.plotting import CONSTANTS_COLOUR_STYLE, override_style, render
from colour.utilities import as_float_array

from colour_hdri.exposure import adjust_exposure

__author__ = 'Colour Developers'
__copyright__ = 'Copyright (C) 2015-2020 - Colour Developers'
__license__ = 'New BSD License - https://opensource.org/licenses/BSD-3-Clause'
__maintainer__ = 'Colour Developers'
__email__ = 'colour-developers@colour-science.org'
__status__ = 'Production'

__all__ = ['plot_radiance_image_strip']


@override_style()
def plot_radiance_image_strip(
        image,
        count=5,
        ev_steps=-2,
        cctf_encoding=CONSTANTS_COLOUR_STYLE.colour.colourspace.cctf_encoding,
        **kwargs):
    """
    Plots given HDRI / radiance image as strip of images of varying exposure.

    Parameters
    ----------
    image : array_like
         HDRI / radiance image to plot.
    count : int, optional
        Strip images count.
    ev_steps : numeric, optional
        Exposure variation for each image of the strip.
    cctf_encoding : callable, optional
        Encoding colour component transfer function / opto-electronic
        transfer function used for plotting.

    Other Parameters
    ----------------
    \\**kwargs : dict, optional
        {:func:`colour.plotting.display`},
        Please refer to the documentation of the previously listed definition.

    Returns
    -------
    tuple
        Current figure and axes.
    """

    image = as_float_array(image)

    grid = matplotlib.gridspec.GridSpec(1, count)
    grid.update(wspace=0, hspace=0)

    height, width, _channel = image.shape
    for i in range(count):
        ev = i * ev_steps
        axis = plt.subplot(grid[i])
        axis.imshow(np.clip(cctf_encoding(adjust_exposure(image, ev)), 0, 1))
        axis.text(
            width * 0.05,
            height - height * 0.05,
            'EV {0}'.format(ev),
            color=(1, 1, 1))
        axis.set_xticks([])
        axis.set_yticks([])
        axis.set_aspect('equal')

    return render(**kwargs)



# -*- coding: utf-8 -*-

from __future__ import absolute_import

from .grossberg2003 import samples_Grossberg2003
from .variance_minimization import (
    light_probe_sampling_variance_minimization_Viriyothai2009)

__all__ = ['samples_Grossberg2003']
__all__ += ['light_probe_sampling_variance_minimization_Viriyothai2009']



# -*- coding: utf-8 -*-
"""
EXIF Data Manipulation
======================

Exif data manipulation routines based on *exiftool*:

-   :func:`colour_hdri.parse_exif_data`
-   :func:`colour_hdri.read_exif_tags`
-   :func:`colour_hdri.copy_exif_tags`
-   :func:`colour_hdri.update_exif_tags`
-   :func:`colour_hdri.delete_exif_tags`
-   :func:`colour_hdri.read_exif_tag`
-   :func:`colour_hdri.write_exif_tag`
"""

from __future__ import division, unicode_literals

import logging
import numpy as np
import re
import subprocess  # nosec
from collections import namedtuple
from fractions import Fraction
from six import text_type

from colour.utilities.documentation import (DocstringText,
                                            is_documentation_building)

from colour_hdri.utilities import vivification

__author__ = 'Colour Developers'
__copyright__ = 'Copyright (C) 2015-2020 - Colour Developers'
__license__ = 'New BSD License - https://opensource.org/licenses/BSD-3-Clause'
__maintainer__ = 'Colour Developers'
__email__ = 'colour-developers@colour-science.org'
__status__ = 'Production'

__all__ = [
    'EXIF_EXECUTABLE', 'ExifTag', 'parse_exif_string', 'parse_exif_numeric',
    'parse_exif_fraction', 'parse_exif_array', 'parse_exif_data',
    'read_exif_tags', 'copy_exif_tags', 'update_exif_tags', 'delete_exif_tags',
    'read_exif_tag', 'write_exif_tag'
]

EXIF_EXECUTABLE = 'exiftool'
if is_documentation_building():  # pragma: no cover
    EXIF_EXECUTABLE = DocstringText(EXIF_EXECUTABLE)
    EXIF_EXECUTABLE.__doc__ = """
Command line exif manipulation application, usually Phil Harvey's *ExifTool*.

EXIF_EXECUTABLE : unicode
"""


class ExifTag(namedtuple('ExifTag', ('group', 'name', 'value', 'identifier'))):
    """
    Hunt colour appearance model induction factors.

    Parameters
    ----------
    group : unicode, optional
        Exif tag group name.
    name : unicode, optional
        Exif tag name.
    value : object, optional
        Exif tag value.
    identifier : numeric, optional
        Exif tag identifier.
    """

    def __new__(cls, group=None, name=None, value=None, identifier=None):
        """
        Returns a new instance of the :class:`colour_hdri.ExifTag` class.
        """

        return super(ExifTag, cls).__new__(cls, group, name, value, identifier)


def parse_exif_string(exif_tag):
    """
    Parses given exif tag assuming it is a string and return its value.

    Parameters
    ----------
    exif_tag : ExifTag
        Exif tag to parse.

    Returns
    -------
    unicode
        Parsed exif tag value.
    """

    return text_type(exif_tag.value)


def parse_exif_numeric(exif_tag, dtype=np.float_):
    """
    Parses given exif tag assuming it is a numeric type and return its value.

    Parameters
    ----------
    exif_tag : ExifTag
        Exif tag to parse.
    dtype : object, optional
        Return value data type.

    Returns
    -------
    numeric
        Parsed exif tag value.
    """

    return dtype(exif_tag.value)


def parse_exif_fraction(exif_tag, dtype=np.float_):
    """
    Parses given exif tag assuming it is a fraction and return its value.

    Parameters
    ----------
    exif_tag : ExifTag
        Exif tag to parse.
    dtype : object, optional
        Return value data type.

    Returns
    -------
    numeric
        Parsed exif tag value.
    """

    return dtype(Fraction(exif_tag.value))


def parse_exif_array(exif_tag, dtype=np.float_, shape=None):
    """
    Parses given exif tag assuming it is an array and return its value.

    Parameters
    ----------
    exif_tag : ExifTag
        Exif tag to parse.
    dtype : object, optional
        Return value data type.
    shape : array_like, optional
        Shape of

    Returns
    -------
    ndarray
        Parsed exif tag value.
    """

    return np.array(exif_tag.value.split()).astype(dtype).reshape(shape)


def parse_exif_data(data):
    """
    Parses given exif data output from *exiftool*.

    Parameters
    ----------
    data : unicode
        Exif data.

    Returns
    -------
    list
        Parsed exif data.
    """

    search = re.search(
        r'\[(?P<group>\w+)\]\s*(?P<id>(\d+|-))?(?P<tag>.*?):(?P<value>.*$)',
        data)

    return [
        group.strip() if group is not None else group
        for group in (search.group('group'), search.group('id'),
                      search.group('tag'), search.group('value'))
    ]


def read_exif_tags(image):
    """
    Returns given image exif image tags.

    Parameters
    ----------
    image : unicode
        Image file.

    Returns
    -------
    defaultdict
        Exif tags.
    """

    logging.info("Reading '{0}' image exif data.".format(image))

    exif_tags = vivification()
    lines = text_type(
        subprocess.check_output(  # nosec
            [EXIF_EXECUTABLE, '-D', '-G', '-a', '-u', '-n', image]),
        'utf-8',
        'ignore').split('\n')

    for line in lines:
        if not line.strip():
            continue

        group, identifier, tag, value = parse_exif_data(line)

        if not exif_tags[group][tag]:
            exif_tags[group][tag] = []

        exif_tags[group][tag].append(ExifTag(group, tag, value, identifier))

    return exif_tags


def copy_exif_tags(source, target):
    """
    Copies given source image file exif tag to given image target.

    Parameters
    ----------
    source : unicode
        Source image file.
    target : unicode
        Target image file.

    Returns
    -------
    bool
        Definition success.
    """

    logging.info("Copying '{0}' file exif data to '{1}' file.".format(
        source, target))

    arguments = [EXIF_EXECUTABLE, '-overwrite_original', '-TagsFromFile']
    arguments += [source, target]
    subprocess.check_output(arguments)  # nosec

    return True


def update_exif_tags(images):
    """
    Updates given images siblings images pairs exif tags.

    Parameters
    ----------
    images : list
        Image files to update.

    Returns
    -------
    bool
        Definition success.
    """

    success = True
    for (source, target) in images:
        success *= copy_exif_tags(source, target)

    return success


def delete_exif_tags(image):
    """
    Deletes all given image exif tags.

    Parameters
    ----------
    image : unicode
        Image file.

    Returns
    -------
    bool
        Definition success.
    """

    logging.info("Deleting '{0}' image exif tags.".format(image))

    subprocess.check_output(  # nosec
        [EXIF_EXECUTABLE, '-overwrite_original', '-all=', image])

    return True


def read_exif_tag(image, tag):
    """
    Returns given image exif tag value.

    Parameters
    ----------
    image : unicode
        Image file.
    tag : unicode
        Tag.

    Returns
    -------
    unicode
        Tag value.
    """

    value = text_type(
        subprocess.check_output(  # nosec
            [EXIF_EXECUTABLE, '-{0}'.format(tag), image]),
        'utf-8',
        'ignore').split(':').pop().strip()

    logging.info("Reading '{0}' image '{1}' exif tag value: '{2}'".format(
        image, tag, value))

    return value


def write_exif_tag(image, tag, value):
    """
    Sets given image exif tag value.

    Parameters
    ----------
    image : unicode
        Image file.
    tag : unicode
        Tag.
    value : unicode
        Value.

    Returns
    -------
    bool
        Definition success.
    """

    logging.info("Writing '{0}' image '{1}' exif tag with '{2}' value.".format(
        image, tag, value))

    arguments = [EXIF_EXECUTABLE, '-overwrite_original']
    arguments += ['-{0}={1}'.format(tag, value), image]
    subprocess.check_output(arguments)  # nosec

    return True



# !/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Defines unit tests for :mod:`colour_hdri.sampling.variance_minimization`
module.
"""

from __future__ import division, unicode_literals

import numpy as np
import os
import unittest

from colour import RGB_COLOURSPACES, RGB_luminance, read_image

from colour_hdri import TESTS_RESOURCES_DIRECTORY
from colour_hdri.sampling import (
    light_probe_sampling_variance_minimization_Viriyothai2009)
from colour_hdri.sampling.variance_minimization import (
    luminance_variance, find_regions_variance_minimization_Viriyothai2009,
    highlight_regions_variance_minimization)

__author__ = 'Colour Developers'
__copyright__ = 'Copyright (C) 2015-2020 - Colour Developers'
__license__ = 'New BSD License - https://opensource.org/licenses/BSD-3-Clause'
__maintainer__ = 'Colour Developers'
__email__ = 'colour-developers@colour-science.org'
__status__ = 'Production'

__all__ = [
    'SAMPLING_DIRECTORY', 'TestLuminanceVariance',
    'TestFindRegionsVarianceMinimizationViriyothai2009',
    'TestLightProbeSamplingVarianceMinimizationViriyothai2009'
]

SAMPLING_DIRECTORY = os.path.join(TESTS_RESOURCES_DIRECTORY, 'colour_hdri',
                                  'sampling')


class TestLuminanceVariance(unittest.TestCase):
    """
    Defines :func:`colour_hdri.sampling.variance_minimization.\
luminance_variance` definition unit tests methods.
    """

    def test_luminance_variance(self):
        """
        Tests :func:`colour_hdri.sampling.variance_minimization.\
luminance_variance` definition.
        """

        a = np.tile(np.arange(5), (5, 1))

        self.assertAlmostEqual(luminance_variance(a), 12.24744871, places=7)


class TestFindRegionsVarianceMinimizationViriyothai2009(unittest.TestCase):
    """
    Defines :func:`colour_hdri.sampling.variance_minimization.\
find_regions_variance_minimization_Viriyothai2009` definition unit tests
    methods.
    """

    def test_find_regions_variance_minimization_Viriyothai2009(self):
        """
        Tests :func:`colour_hdri.sampling.variance_minimization.\
find_regions_variance_minimization_Viriyothai2009` definition.
        """

        colourspace = RGB_COLOURSPACES['sRGB']

        image = read_image(
            str(
                os.path.join(
                    SAMPLING_DIRECTORY,
                    'tests_light_probe_sampling_variance_minimization_'
                    'Viriyothai2009.exr')))

        Y = RGB_luminance(image, colourspace.primaries, colourspace.whitepoint)

        regions = find_regions_variance_minimization_Viriyothai2009(Y, n=1)
        self.assertListEqual(
            regions,
            [(0, 256, 0, 156),
             (0, 256, 156, 256)])  # yapf: disable

        regions = find_regions_variance_minimization_Viriyothai2009(Y, n=2)
        self.assertListEqual(
            regions,
            [(0, 97, 0, 156),
             (97, 256, 0, 156),
             (0, 100, 156, 256),
             (100, 256, 156, 256)])  # yapf: disable

        regions = find_regions_variance_minimization_Viriyothai2009(Y, n=4)
        self.assertListEqual(
            regions,
            [(0, 39, 0, 91),
             (39, 97, 0, 91),
             (0, 39, 91, 156),
             (39, 97, 91, 156),
             (97, 159, 0, 92),
             (97, 159, 92, 156),
             (159, 256, 0, 93),
             (159, 256, 93, 156),
             (0, 42, 156, 216),
             (42, 100, 156, 216),
             (0, 44, 216, 256),
             (44, 100, 216, 256),
             (100, 163, 156, 215),
             (100, 163, 215, 256),
             (163, 256, 156, 216),
             (163, 256, 216, 256)])  # yapf: disable


class TestHighlightRegionsVarianceMinimization(unittest.TestCase):
    """
    Defines :func:`colour_hdri.sampling.variance_minimization.\
highlight_regions_variance_minimization` definition unit tests methods.
    """

    def test_highlight_regions_variance_minimization(self):
        """
        Tests :func:`colour_hdri.sampling.variance_minimization.\
highlight_regions_variance_minimization` definition.
        """

        colourspace = RGB_COLOURSPACES['sRGB']

        image = read_image(
            str(
                os.path.join(
                    SAMPLING_DIRECTORY,
                    'tests_light_probe_sampling_variance_minimization_'
                    'Viriyothai2009.exr')))

        Y = RGB_luminance(image, colourspace.primaries, colourspace.whitepoint)
        regions = find_regions_variance_minimization_Viriyothai2009(Y, n=4)
        np.testing.assert_almost_equal(
            highlight_regions_variance_minimization(image, regions),
            read_image(
                str(
                    os.path.join(
                        SAMPLING_DIRECTORY,
                        'tests_highlight_regions_variance_minimization.exr'))),
            decimal=7)


class TestLightProbeSamplingVarianceMinimizationViriyothai2009(
        unittest.TestCase):
    """
    Defines :func:`colour_hdri.sampling.variance_minimization.\
light_probe_sampling_variance_minimization_Viriyothai2009` definition unit
    tests methods.
    """

    def test_light_probe_sampling_variance_minimization_Viriyothai2009(self):
        """
        Tests :func:`colour_hdri.sampling.variance_minimization.\
light_probe_sampling_variance_minimization_Viriyothai2009` definition.
        """

        image = read_image(
            str(
                os.path.join(
                    SAMPLING_DIRECTORY,
                    'tests_light_probe_sampling_variance_minimization_'
                    'Viriyothai2009.exr')))

        lights = light_probe_sampling_variance_minimization_Viriyothai2009(
            image)
        uvs = np.array([light[0] for light in lights])
        colours = np.array([light[1] for light in lights])
        indexes = np.array([light[2] for light in lights])

        np.testing.assert_almost_equal(
            uvs,
            np.array(
                [[0.16015625, 0.11328125],
                 [0.15625000, 0.32421875],
                 [0.42968750, 0.11328125],
                 [0.42968750, 0.32031250],
                 [0.16796875, 0.48437500],
                 [0.43359375, 0.48437500],
                 [0.18359375, 0.74218750],
                 [0.44140625, 0.75390625],
                 [0.64843750, 0.11718750],
                 [0.64843750, 0.35156250],
                 [0.87500000, 0.11718750],
                 [0.87500000, 0.35937500],
                 [0.64062500, 0.54687500],
                 [0.87500000, 0.54687500],
                 [0.64843750, 0.79687500],
                 [0.87500000, 0.80078125]]),
            decimal=7)  # yapf: disable

        np.testing.assert_almost_equal(
            colours,
            np.array(
                [[40.65992016, 226.11660475, 266.78098774],
                 [30.73776919, 130.37145448, 161.10773420],
                 [98.10281688, 201.29676312, 299.40549910],
                 [74.89445847, 117.00525796, 191.89859456],
                 [42.24291545, 125.58142895, 167.82099468],
                 [90.15780473, 102.82409275, 192.97436064],
                 [82.17253280, 97.55175847, 179.72394174],
                 [152.10083479, 72.06310755, 224.16233468],
                 [128.44782221, 173.29928458, 301.74819988],
                 [105.67531514, 104.05130512, 209.73328489],
                 [196.73262107, 196.16986090, 392.89716852],
                 [154.23001331, 111.56532115, 265.79564852],
                 [120.04539376, 81.22123903, 201.26542896],
                 [191.57947493, 95.21154106, 286.79548484],
                 [168.29435712, 45.09299320, 213.38641733],
                 [253.65272349, 50.30476046, 303.96245855]]),
            decimal=7)  # yapf: disable

        np.testing.assert_array_equal(
            indexes,
            np.array(
                [[29, 41],
                 [83, 40],
                 [29, 110],
                 [82, 110],
                 [124, 43],
                 [124, 111],
                 [190, 47],
                 [193, 113],
                 [30, 166],
                 [90, 166],
                 [30, 224],
                 [92, 224],
                 [140, 164],
                 [140, 224],
                 [204, 166],
                 [205, 224]]))  # yapf: disable


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
"""
Image Data & Metadata Utilities
===============================

Defines various image data and metadata utilities classes:

-   :class:`colour_hdri.Metadata`
-   :class:`colour_hdri.Image`
-   :class:`colour_hdri.ImageStack`
"""

from __future__ import division, unicode_literals

import logging
import numpy as np
from collections import MutableSequence
from recordclass import recordclass

from colour import read_image
from colour.utilities import as_float_array, is_string, tsplit, tstack, warning

from colour_hdri.exposure import average_luminance
from colour_hdri.utilities.exif import (parse_exif_array, parse_exif_fraction,
                                        parse_exif_numeric, read_exif_tags)

__author__ = 'Colour Developers'
__copyright__ = 'Copyright (C) 2015-2020 - Colour Developers'
__license__ = 'New BSD License - https://opensource.org/licenses/BSD-3-Clause'
__maintainer__ = 'Colour Developers'
__email__ = 'colour-developers@colour-science.org'
__status__ = 'Production'

__all__ = ['Metadata', 'Image', 'ImageStack']


class Metadata(
        recordclass('Metadata',
                    ('f_number', 'exposure_time', 'iso', 'black_level',
                     'white_level', 'white_balance_multipliers'))):
    """
    Defines the base object for storing exif metadata relevant to
    HDRI / radiance image generation.

    Parameters
    ----------
    f_number : array_like
        Image *FNumber*.
    exposure_time : array_like
        Image *Exposure Time*.
    iso : array_like
        Image *ISO*.
    black_level : array_like
        Image *Black Level*.
    white_level : array_like
        Image *White Level*.
    white_balance_multipliers : array_like
        Image white balance multipliers, usually the *As Shot Neutral*  matrix.
    """

    def __new__(cls,
                f_number,
                exposure_time,
                iso,
                black_level=None,
                white_level=None,
                white_balance_multipliers=None):
        return super(Metadata, cls).__new__(cls, f_number, exposure_time, iso,
                                            black_level, white_level,
                                            white_balance_multipliers)


class Image(object):
    """
    Defines the base object for storing an image along its path, pixel data and
    metadata needed for HDRI / radiance images generation.

    Parameters
    ----------
    path : unicode, optional
        Image path.
    data : array_like, optional
        Image pixel data array.
    metadata : Metadata, optional
        Image exif metadata.

    Attributes
    ----------
    -   :attr:`colour_hdri.Image.path`
    -   :attr:`colour_hdri.Image.data`
    -   :attr:`colour_hdri.Image.metadata`

    Methods
    -------
    -   :meth:`colour_hdri.Image.__init__`
    -   :meth:`colour_hdri.Image.read_data`
    -   :meth:`colour_hdri.Image.read_metadata`
    """

    def __init__(self, path=None, data=None, metadata=None):
        self._path = None
        self.path = path
        self._data = None
        self.data = data
        self._metadata = None
        self.metadata = metadata

    @property
    def path(self):
        """
        Property for **self._path** private attribute.

        Returns
        -------
        unicode
            self._path.
        """

        return self._path

    @path.setter
    def path(self, value):
        """
        Setter for **self._path** private attribute.

        Parameters
        ----------
        value : unicode
            Attribute value.
        """

        if value is not None:
            assert is_string(value), (('"{0}" attribute: "{1}" is not a '
                                       '"string" like object!').format(
                                           'path', value))

        self._path = value

    @property
    def data(self):
        """
        Property for **self._data** private attribute.

        Returns
        -------
        unicode
            self._data.
        """

        return self._data

    @data.setter
    def data(self, value):
        """
        Setter for **self._data** private attribute.

        Parameters
        ----------
        value : unicode
            Attribute value.
        """

        if value is not None:
            assert isinstance(value, (tuple, list, np.ndarray, np.matrix)), ((
                '"{0}" attribute: "{1}" is not a "tuple", "list", "ndarray" '
                'or "matrix" instance!').format('data', value))

        self._data = as_float_array(value)

    @property
    def metadata(self):
        """
        Property for **self._metadata** private attribute.

        Returns
        -------
        unicode
            self._metadata.
        """

        return self._metadata

    @metadata.setter
    def metadata(self, value):
        """
        Setter for **self._metadata** private attribute.

        Parameters
        ----------
        value : unicode
            Attribute value.
        """

        if value is not None:
            assert isinstance(value, Metadata), (
                '"{0}" attribute: "{1}" is not a "Metadata" instance!'.format(
                    'metadata', value))

        self._metadata = value

    def read_data(self, cctf_decoding=None):
        """
        Reads image pixel data at :attr:`Image.path` attribute.

        Parameters
        ----------
        cctf_decoding : object, optional
            Decoding colour component transfer function (Decoding CCTF) or
            electro-optical transfer function (EOTF / EOCF).

        Returns
        -------
        ndarray
            Image pixel data.
        """

        logging.info('Reading "{0}" image.'.format(self._path))
        self.data = read_image(str(self._path))
        if cctf_decoding is not None:
            self.data = cctf_decoding(self.data)

        return self.data

    def read_metadata(self):
        """
        Reads image relevant exif metadata at :attr:`Image.path` attribute.

        Returns
        -------
        Metadata
            Image relevant exif metadata.
        """

        logging.info('Reading "{0}" image metadata.'.format(self._path))
        exif_data = read_exif_tags(self._path)

        if not exif_data.get('EXIF'):
            warning(
                '"{0}" file has no "Exif" data, metadata will be undefined!'.
                format(self._path))
            self.metadata = Metadata(*[None] * 6)
            return self.metadata

        f_number = exif_data['EXIF'].get('F Number')
        if f_number is not None:
            f_number = parse_exif_numeric(f_number[0])

        exposure_time = exif_data['EXIF'].get('Exposure Time')
        if exposure_time is not None:
            exposure_time = parse_exif_fraction(exposure_time[0])

        iso = exif_data['EXIF'].get('ISO')
        if iso is not None:
            iso = parse_exif_numeric(iso[0])

        black_level = exif_data['EXIF'].get('Black Level')
        if black_level is not None:
            black_level = parse_exif_array(black_level[0])
            black_level = as_float_array(black_level) / 65535

        white_level = exif_data['EXIF'].get('White Level')
        if white_level is not None:
            white_level = parse_exif_array(white_level[0])
            white_level = as_float_array(white_level) / 65535

        white_balance_multipliers = exif_data['EXIF'].get('As Shot Neutral')
        if white_balance_multipliers is not None:
            white_balance_multipliers = parse_exif_array(
                white_balance_multipliers[0])
            white_balance_multipliers = as_float_array(
                white_balance_multipliers) / white_balance_multipliers[1]

        self.metadata = Metadata(f_number, exposure_time, iso, black_level,
                                 white_level, white_balance_multipliers)

        return self.metadata


class ImageStack(MutableSequence):
    """
    Defines a convenient stack storing a sequence of images for HDRI / radiance
    images generation.

    Methods
    -------
    -   :meth:`colour_hdri.ImageStack.__init__`
    -   :meth:`colour_hdri.ImageStack.__getitem__`
    -   :meth:`colour_hdri.ImageStack.__setitem__`
    -   :meth:`colour_hdri.ImageStack.__delitem__`
    -   :meth:`colour_hdri.ImageStack.__len__`
    -   :meth:`colour_hdri.ImageStack.__getattr__`
    -   :meth:`colour_hdri.ImageStack.__setattr__`
    -   :meth:`colour_hdri.ImageStack.sort`
    -   :meth:`colour_hdri.ImageStack.insert`
    -   :meth:`colour_hdri.ImageStack.from_files`
    """

    def __init__(self):
        self._list = []

    def __getitem__(self, index):
        """
        Reimplements the :meth:`MutableSequence.__getitem__` method.

        Parameters
        ----------
        index : int
            Item index.

        Returns
        -------
        Image
            Item at given index.
        """

        return self._list[index]

    def __setitem__(self, index, value):
        """
        Reimplements the :meth:`MutableSequence.__setitem__` method.

        Parameters
        ----------
        index : int
            Item index.
        value : int
            Item value.
        """

        self._list[index] = value

    def __delitem__(self, index):
        """
        Reimplements the :meth:`MutableSequence.__delitem__` method.

        Parameters
        ----------
        index : int
            Item index.
        """

        del self._list[index]

    def __len__(self):
        """
        Reimplements the :meth:`MutableSequence.__len__` method.
        """

        return len(self._list)

    def __getattr__(self, attribute):
        """
        Reimplements the :meth:`MutableSequence.__getattr__` method.

        Parameters
        ----------
        attribute : unicode
            Attribute to retrieve the value.

        Returns
        -------
        object
            Attribute value.
        """

        try:
            return self.__dict__[attribute]
        except KeyError:
            if hasattr(Image, attribute):
                value = [getattr(image, attribute) for image in self]
                if attribute == 'data':
                    return tstack(value)
                else:
                    return tuple(value)
            elif hasattr(Metadata, attribute):
                value = [getattr(image.metadata, attribute) for image in self]
                return as_float_array(value)
            else:
                raise AttributeError(
                    "'{0}' object has no attribute '{1}'".format(
                        self.__class__.__name__, attribute))

    def __setattr__(self, attribute, value):
        """
        Reimplements the :meth:`MutableSequence.__getattr__` method.

        Parameters
        ----------
        attribute : unicode
            Attribute to set the value.
        value : object
            Value to set.
        """

        if hasattr(Image, attribute):
            if attribute == 'data':
                data = tsplit(value)
                for i, image in enumerate(self):
                    image.data = data[i]
            else:
                for i, image in enumerate(self):
                    setattr(image, attribute, value[i])
        elif hasattr(Metadata, attribute):
            for i, image in enumerate(self):
                setattr(image.metadata, attribute, value[i])
        else:
            super(ImageStack, self).__setattr__(attribute, value)

    def insert(self, index, value):
        """
        Reimplements the :meth:`MutableSequence.insert` method.

        Parameters
        ----------
        index : int
            Item index.
        value : object
            Item value.
        """

        self._list.insert(index, value)

    def sort(self, key=None):
        """
        Sorts the underlying data structure.

        Parameters
        ----------
        key : callable
            Function of one argument that is used to extract a comparison key
            from each data structure.
        """

        self._list = sorted(self._list, key=key)

    @staticmethod
    def from_files(image_files, cctf_decoding=None):
        """
        Returns a :class:`colour_hdri.ImageStack` instance with given image
        files.

        Parameters
        ----------
        image_files : array_like
            Image files.
        cctf_decoding : object, optional
            Decoding colour component transfer function (Decoding CCTF) or
            electro-optical transfer function (EOTF / EOCF).

        Returns
        -------
        ImageStack
        """

        image_stack = ImageStack()
        for image_file in image_files:
            image = Image(image_file)
            image.read_data(cctf_decoding)
            image.read_metadata()
            image_stack.append(image)

        def luminance_average_key(image):
            """
            Comparison key function.
            """

            f_number = image.metadata.f_number
            exposure_time = image.metadata.exposure_time
            iso = image.metadata.iso

            if None in (f_number, exposure_time, iso):
                warning('"{0}" exposure data is missing, average luminance '
                        'sorting is inapplicable!'.format(image.path))
                return None

            return 1 / average_luminance(f_number, exposure_time, iso)

        image_stack.sort(luminance_average_key)

        return image_stack



# -*- coding: utf-8 -*-
"""
Colour - HDRI
=============

HDRI - Radiance image processing algorithms for *Python*.

Subpackages
-----------
-   calibration: Camera calibration computations.
-   exposure: Exposure computations.
-   examples: Examples for the sub-packages.
-   generation: HDRI / radiance image generation.
-   models: Colour models conversion.
-   plotting: Diagrams, figures, etc...
-   process: Image conversion helpers.
-   recovery: Clipped highlights recovery.
-   resources: Resources sub-modules.
-   sampling: Image sampling routines.
-   tonemapping: Tonemapping operators.
-   utilities: Various utilities and data structures.
"""

from __future__ import absolute_import

import numpy as np
import os
import recordclass
import subprocess  # nosec

import colour

from .utilities import (EXIF_EXECUTABLE, ExifTag, Image, ImageStack, Metadata,
                        copy_exif_tags, delete_exif_tags, filter_files,
                        parse_exif_array, parse_exif_data, parse_exif_fraction,
                        parse_exif_numeric, parse_exif_string, path_exists,
                        read_exif_tag, read_exif_tags, update_exif_tags,
                        vivification, vivified_to_dict, write_exif_tag)
from .sampling import (
    light_probe_sampling_variance_minimization_Viriyothai2009,
    samples_Grossberg2003)
from .exposure import (
    adjust_exposure, arithmetic_mean_focal_plane_exposure, average_illuminance,
    average_luminance, exposure_index_values, exposure_value_100,
    photometric_exposure_scale_factor_Lagarde2014, focal_plane_exposure,
    illuminance_to_exposure_value, luminance_to_exposure_value,
    saturation_based_speed_focal_plane_exposure)
from .generation import (normal_distribution_function, hat_function,
                         weighting_function_Debevec1997,
                         image_stack_to_radiance_image)
from .calibration import (absolute_luminance_calibration_Lagarde2016,
                          camera_response_functions_Debevec1997, g_solve,
                          upper_hemisphere_illuminance_weights_Lagarde2016)
from .models import (camera_neutral_to_xy, camera_space_to_RGB,
                     camera_space_to_sRGB, camera_space_to_XYZ_matrix,
                     xy_to_camera_neutral, XYZ_to_camera_space_matrix)
from .process import (DNG_CONVERSION_ARGUMENTS, DNG_CONVERTER,
                      DNG_EXIF_TAGS_BINDING, RAW_CONVERSION_ARGUMENTS,
                      RAW_CONVERTER, RAW_D_CONVERSION_ARGUMENTS,
                      convert_dng_files_to_intermediate_files,
                      convert_raw_files_to_dng_files, read_dng_files_exif_tags)
from .recovery import highlights_recovery_blend, highlights_recovery_LCHab
from .tonemapping import (
    tonemapping_operator_exponential,
    tonemapping_operator_exponentiation_mapping, tonemapping_operator_filmic,
    tonemapping_operator_gamma, tonemapping_operator_logarithmic,
    tonemapping_operator_logarithmic_mapping,
    tonemapping_operator_normalisation, tonemapping_operator_Reinhard2004,
    tonemapping_operator_Schlick1994, tonemapping_operator_simple,
    tonemapping_operator_Tumblin1999)

__author__ = 'Colour Developers'
__copyright__ = 'Copyright (C) 2015-2020 - Colour Developers'
__license__ = 'New BSD License - https://opensource.org/licenses/BSD-3-Clause'
__maintainer__ = 'Colour Developers'
__email__ = 'colour-developers@colour-science.org'
__status__ = 'Production'

__all__ = [
    'EXIF_EXECUTABLE', 'ExifTag', 'Image', 'ImageStack', 'Metadata',
    'copy_exif_tags', 'delete_exif_tags', 'filter_files', 'parse_exif_array',
    'parse_exif_data', 'parse_exif_fraction', 'parse_exif_numeric',
    'parse_exif_string', 'path_exists', 'read_exif_tag', 'read_exif_tags',
    'update_exif_tags', 'vivification', 'vivified_to_dict', 'write_exif_tag'
]
__all__ += [
    'light_probe_sampling_variance_minimization_Viriyothai2009',
    'samples_Grossberg2003'
]
__all__ += [
    'adjust_exposure', 'arithmetic_mean_focal_plane_exposure',
    'average_illuminance', 'average_luminance', 'exposure_index_values',
    'exposure_value_100', 'photometric_exposure_scale_factor_Lagarde2014',
    'focal_plane_exposure', 'illuminance_to_exposure_value',
    'luminance_to_exposure_value',
    'saturation_based_speed_focal_plane_exposure'
]
__all__ += [
    'normal_distribution_function', 'hat_function',
    'weighting_function_Debevec1997', 'image_stack_to_radiance_image'
]
__all__ += [
    'absolute_luminance_calibration_Lagarde2016',
    'camera_response_functions_Debevec1997', 'g_solve',
    'upper_hemisphere_illuminance_weights_Lagarde2016'
]
__all__ += [
    'camera_neutral_to_xy', 'camera_space_to_RGB', 'camera_space_to_sRGB',
    'camera_space_to_XYZ_matrix', 'xy_to_camera_neutral',
    'XYZ_to_camera_space_matrix'
]
__all__ += [
    'DNG_CONVERSION_ARGUMENTS', 'DNG_CONVERTER', 'DNG_EXIF_TAGS_BINDING',
    'RAW_CONVERSION_ARGUMENTS', 'RAW_CONVERTER', 'RAW_D_CONVERSION_ARGUMENTS',
    'convert_dng_files_to_intermediate_files',
    'convert_raw_files_to_dng_files', 'read_dng_files_exif_tags'
]
__all__ += ['highlights_recovery_blend', 'highlights_recovery_LCHab']
__all__ += [
    'tonemapping_operator_exponential',
    'tonemapping_operator_exponentiation_mapping',
    'tonemapping_operator_filmic', 'tonemapping_operator_gamma',
    'tonemapping_operator_logarithmic',
    'tonemapping_operator_logarithmic_mapping',
    'tonemapping_operator_normalisation', 'tonemapping_operator_Reinhard2004',
    'tonemapping_operator_Schlick1994', 'tonemapping_operator_simple',
    'tonemapping_operator_Tumblin1999'
]

RESOURCES_DIRECTORY = os.path.join(os.path.dirname(__file__), 'resources')
EXAMPLES_RESOURCES_DIRECTORY = os.path.join(RESOURCES_DIRECTORY,
                                            'colour-hdri-examples-datasets')
TESTS_RESOURCES_DIRECTORY = os.path.join(RESOURCES_DIRECTORY,
                                         'colour-hdri-tests-datasets')

__application_name__ = 'Colour - HDRI'

__major_version__ = '0'
__minor_version__ = '1'
__change_version__ = '8'
__version__ = '.'.join(
    (__major_version__,
     __minor_version__,
     __change_version__))  # yapf: disable

try:
    version = subprocess.check_output(  # nosec
        ['git', 'describe'],
        cwd=os.path.dirname(__file__),
        stderr=subprocess.STDOUT).strip()
    version = version.decode('utf-8')
except Exception:
    version = __version__

colour.utilities.ANCILLARY_COLOUR_SCIENCE_PACKAGES['colour-hdri'] = version
colour.utilities.ANCILLARY_RUNTIME_PACKAGES[
    'recordclass'] = recordclass.__version__

# TODO: Remove legacy printing support when deemed appropriate.
try:
    np.set_printoptions(legacy='1.13')
except TypeError:
    pass



# !/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Defines unit tests for :mod:`colour_hdri.recovery.highlights` module.
"""

from __future__ import division, unicode_literals

import numpy as np
import os
import platform
import re
import shlex
import shutil
import subprocess  # nosec
import tempfile
import unittest

from colour import read_image

from colour_hdri import TESTS_RESOURCES_DIRECTORY
from colour_hdri.process import RAW_CONVERTER, RAW_D_CONVERSION_ARGUMENTS
from colour_hdri.recovery import (highlights_recovery_blend,
                                  highlights_recovery_LCHab)
from colour_hdri.models import camera_space_to_sRGB
from colour_hdri.utilities import filter_files

__author__ = 'Colour Developers'
__copyright__ = 'Copyright (C) 2015-2020 - Colour Developers'
__license__ = 'New BSD License - https://opensource.org/licenses/BSD-3-Clause'
__maintainer__ = 'Colour Developers'
__email__ = 'colour-developers@colour-science.org'
__status__ = 'Production'

__all__ = [
    'FROBISHER_001_DIRECTORY', 'RECOVERY_DIRECTORY', 'RAW_IMAGES',
    'XYZ_TO_CAMERA_SPACE_MATRIX', 'TestHighlightsRecoveryBlend',
    'TestHighlightsRecoveryLCHab'
]

_IS_WINDOWS_PLATFORM = platform.system() in ('Windows', 'Microsoft')
"""
Whether the current platform is *Windows*.
"""

FROBISHER_001_DIRECTORY = os.path.join(TESTS_RESOURCES_DIRECTORY,
                                       'frobisher_001')

RECOVERY_DIRECTORY = os.path.join(TESTS_RESOURCES_DIRECTORY, 'colour_hdri',
                                  'recovery')

RAW_IMAGES = filter_files(FROBISHER_001_DIRECTORY, ('CR2', ))

XYZ_TO_CAMERA_SPACE_MATRIX = np.array(
    [[0.47160000, 0.06030000, -0.08300000],
     [-0.77980000, 1.54740000, 0.24800000],
     [-0.14960000, 0.19370000, 0.66510000]])  # yapf: disable


class TestHighlightsRecoveryBlend(unittest.TestCase):
    """
    Defines :func:`colour_hdri.recovery.highlights.\
highlights_recovery_blend` definition unit tests methods.
    """

    def setUp(self):
        """
        Initialises common tests attributes.
        """

        self._temporary_directory = tempfile.mkdtemp()

    def tearDown(self):
        """
        After tests actions.
        """

        shutil.rmtree(self._temporary_directory)

    def test_highlights_recovery_blend(self):
        """
        Tests :func:`colour_hdri.recovery.highlights.highlights_recovery_blend`
        definition.
        """

        multipliers = np.array([2.42089718, 1.00000000, 1.54687415])
        multipliers /= np.max(multipliers)

        reference_raw_file = RAW_IMAGES[1]
        test_raw_file = os.path.join(self._temporary_directory,
                                     os.path.basename(reference_raw_file))
        shutil.copyfile(reference_raw_file, test_raw_file)
        command = [RAW_CONVERTER] + shlex.split(
            RAW_D_CONVERSION_ARGUMENTS.format(test_raw_file),
            posix=not _IS_WINDOWS_PLATFORM)

        subprocess.call(command)  # nosec

        test_tiff_file = read_image(
            str(re.sub('\\.CR2$', '.tiff', test_raw_file)))[::10, ::10, :]

        test_tiff_file *= multipliers
        test_tiff_file = highlights_recovery_blend(test_tiff_file, multipliers)
        test_tiff_file = camera_space_to_sRGB(test_tiff_file,
                                              XYZ_TO_CAMERA_SPACE_MATRIX)
        reference_exr_path = os.path.join(
            RECOVERY_DIRECTORY,
            os.path.basename(re.sub('\\.CR2$', '_Blend.exr', test_raw_file)))
        reference_exr_file = read_image(str(reference_exr_path))

        np.testing.assert_allclose(
            test_tiff_file, reference_exr_file, rtol=0.0001, atol=0.0001)


class TestHighlightsRecoveryLCHab(unittest.TestCase):
    """
    Defines :func:`colour_hdri.recovery.highlights.\
highlights_recovery_LCHab` definition unit tests methods.
    """

    def setUp(self):
        """
        Initialises common tests attributes.
        """

        self._temporary_directory = tempfile.mkdtemp()

    def tearDown(self):
        """
        After tests actions.
        """

        shutil.rmtree(self._temporary_directory)

    def test_highlights_recovery_LCHab(self):
        """
        Tests :func:`colour_hdri.recovery.highlights.highlights_recovery_LCHab`
        definition.
        """

        multipliers = np.array([2.42089718, 1.00000000, 1.54687415])
        multipliers /= np.max(multipliers)

        reference_raw_file = RAW_IMAGES[1]
        test_raw_file = os.path.join(self._temporary_directory,
                                     os.path.basename(reference_raw_file))
        shutil.copyfile(reference_raw_file, test_raw_file)
        command = [RAW_CONVERTER] + shlex.split(
            RAW_D_CONVERSION_ARGUMENTS.format(test_raw_file),
            posix=not _IS_WINDOWS_PLATFORM)

        subprocess.call(command)  # nosec

        test_tiff_file = read_image(
            str(re.sub('\\.CR2$', '.tiff', test_raw_file)))[::10, ::10, :]

        test_tiff_file *= multipliers
        test_tiff_file = highlights_recovery_LCHab(test_tiff_file,
                                                   min(multipliers))
        test_tiff_file = camera_space_to_sRGB(test_tiff_file,
                                              XYZ_TO_CAMERA_SPACE_MATRIX)

        reference_exr_path = os.path.join(
            RECOVERY_DIRECTORY,
            os.path.basename(re.sub('\\.CR2$', '_LCHab.exr', test_raw_file)))
        reference_exr_file = read_image(str(reference_exr_path))

        np.testing.assert_allclose(
            test_tiff_file, reference_exr_file, rtol=0.0001, atol=0.0001)


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-

from __future__ import absolute_import

from .common import (average_luminance, average_illuminance,
                     luminance_to_exposure_value,
                     illuminance_to_exposure_value, adjust_exposure)
from .dsc import (focal_plane_exposure, arithmetic_mean_focal_plane_exposure,
                  saturation_based_speed_focal_plane_exposure,
                  exposure_index_values, exposure_value_100,
                  photometric_exposure_scale_factor_Lagarde2014)

__all__ = [
    'average_luminance', 'average_illuminance', 'luminance_to_exposure_value',
    'illuminance_to_exposure_value', 'adjust_exposure'
]
__all__ += [
    'focal_plane_exposure', 'arithmetic_mean_focal_plane_exposure',
    'saturation_based_speed_focal_plane_exposure', 'exposure_index_values',
    'exposure_value_100', 'photometric_exposure_scale_factor_Lagarde2014'
]



# -*- coding: utf-8 -*-
"""
Common Utilities
================

Defines common utilities objects that don't fall in any specific category.
"""

from __future__ import division, unicode_literals

import os
import re

from collections import defaultdict

__author__ = 'Colour Developers'
__copyright__ = 'Copyright (C) 2015-2020 - Colour Developers'
__license__ = 'New BSD License - https://opensource.org/licenses/BSD-3-Clause'
__maintainer__ = 'Colour Developers'
__email__ = 'colour-developers@colour-science.org'
__status__ = 'Production'

__all__ = ['vivification', 'vivified_to_dict', 'path_exists', 'filter_files']


def vivification():
    """
    Implements supports for vivification of the underlying dict like
    data-structure, magical!

    Returns
    -------
    defaultdict

    Examples
    --------
    >>> vivified = vivification()
    >>> vivified['my']['attribute'] = 1
    >>> vivified['my']  # doctest: +SKIP
    defaultdict(<function vivification at 0x...>, {u'attribute': 1})
    >>> vivified['my']['attribute']
    1
    """

    return defaultdict(vivification)


def vivified_to_dict(vivified):
    """
    Converts given vivified data-structure to dictionary.

    Parameters
    ----------
    vivified : defaultdict
        Vivified data-structure.

    Returns
    -------
    dict

    Examples
    --------
    >>> vivified = vivification()
    >>> vivified['my']['attribute'] = 1
    >>> vivified_to_dict(vivified)  # doctest: +SKIP
    {u'my': {u'attribute': 1}}
    """

    if isinstance(vivified, defaultdict):
        vivified = {
            key: vivified_to_dict(value)
            for key, value in vivified.items()
        }
    return vivified


def path_exists(path):
    """
    Returns if given path exists.

    Parameters
    ----------
    path : unicode
        Path to check the existence.

    Returns
    -------
    bool

    Examples
    --------
    >>> path_exists(__file__)
    True
    >>> path_exists('')
    False
    """

    if not path:
        return False
    else:
        return os.path.exists(path)


def filter_files(directory, extensions):
    """
    Filters given directory for files matching given extensions.

    Parameters
    ----------
    directory : unicode
        Directory to filter.
    extensions : tuple or list
        Extensions to filter on.

    Returns
    -------
    list
        Filtered files.
    """

    return [
        os.path.join(directory, path) for path in filter(
            lambda x: re.search('{0}$'.format('|'.join(extensions)), x),
            sorted(os.listdir(directory)))
    ]



# -*- coding: utf-8 -*-

from __future__ import absolute_import

from .datasets import *  # noqa
from . import datasets
from .dng import (xy_to_camera_neutral, camera_neutral_to_xy,
                  XYZ_to_camera_space_matrix, camera_space_to_XYZ_matrix)
from .rgb import camera_space_to_RGB, camera_space_to_sRGB

__all__ = []
__all__ += datasets.__all__
__all__ += [
    'xy_to_camera_neutral', 'camera_neutral_to_xy',
    'XYZ_to_camera_space_matrix', 'camera_space_to_XYZ_matrix'
]
__all__ += ['camera_space_to_RGB', 'camera_space_to_sRGB']



# !/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Defines unit tests for :mod:`colour_hdri.models.dng` module.

Notes
-----
-   The various matrices used in this module are extracted from a
    *Canon EOS 5D Mark II* camera.
"""

from __future__ import division, unicode_literals

import numpy as np
import unittest

from colour_hdri.models.dng import interpolated_matrix
from colour_hdri.models import (xy_to_camera_neutral, camera_neutral_to_xy,
                                XYZ_to_camera_space_matrix,
                                camera_space_to_XYZ_matrix)

__author__ = 'Colour Developers'
__copyright__ = 'Copyright (C) 2015-2020 - Colour Developers'
__license__ = 'New BSD License - https://opensource.org/licenses/BSD-3-Clause'
__maintainer__ = 'Colour Developers'
__email__ = 'colour-developers@colour-science.org'
__status__ = 'Production'

__all__ = [
    'CCT_CALIBRATION_ILLUMINANT_1', 'CCT_CALIBRATION_ILLUMINANT_2',
    'M_COLOR_MATRIX_1', 'M_COLOR_MATRIX_2', 'M_CAMERA_CALIBRATION_1',
    'M_CAMERA_CALIBRATION_2', 'ANALOG_BALANCE', 'M_FORWARD_MATRIX_1',
    'M_FORWARD_MATRIX_2', 'TestInterpolatedMatrix', 'TestXy_to_camera_neutral',
    'TestCamera_neutral_to_xy', 'TestXYZ_to_camera_space_matrix',
    'TestCamera_space_to_XYZ_matrix'
]

CCT_CALIBRATION_ILLUMINANT_1 = 2850
CCT_CALIBRATION_ILLUMINANT_2 = 6500

M_COLOR_MATRIX_1 = np.array(
    [[0.5309, -0.0229, -0.0336],
     [-0.6241, 1.3265, 0.3337],
     [-0.0817, 0.1215, 0.6664]])  # yapf: disable
M_COLOR_MATRIX_2 = np.array(
    [[0.4716, 0.0603, -0.0830],
     [-0.7798, 1.5474, 0.2480],
     [-0.1496, 0.1937, 0.6651]])  # yapf: disable

M_CAMERA_CALIBRATION_1 = np.array(
    [[0.9603, 0.0000, 0.0000],
     [0.0000, 1.0000, 0.0000],
     [0.0000, 0.0000, 0.9664]])  # yapf: disable
M_CAMERA_CALIBRATION_2 = np.array(
    [[0.9603, 0.0000, 0.0000],
     [0.0000, 1.0000, 0.0000],
     [0.0000, 0.0000, 0.9664]])  # yapf: disable

ANALOG_BALANCE = np.ones(3)

M_FORWARD_MATRIX_1 = np.array(
    [[0.8924, -0.1041, 0.1760],
     [0.4351, 0.6621, -0.0972],
     [0.0505, -0.1562, 0.9308]])  # yapf: disable
M_FORWARD_MATRIX_2 = np.array(
    [[0.8924, -0.1041, 0.1760],
     [0.4351, 0.6621, -0.0972],
     [0.0505, -0.1562, 0.9308]])  # yapf: disable


class TestInterpolatedMatrix(unittest.TestCase):
    """
    Defines :func:`colour_hdri.models.adobe_dng.\
interpolated_matrix` definition unit tests methods.
    """

    def test_interpolated_matrix(self):
        """
        Tests :func:`colour_hdri.models.adobe_dng.\
interpolated_matrix` definition.
        """

        M_reference = np.array(
            [[0.48549082, 0.04081068, -0.07142822],
             [-0.74332781, 1.49565493, 0.26807493],
             [-0.13369466, 0.17678740, 0.66540452]])  # yapf: disable
        np.testing.assert_almost_equal(
            interpolated_matrix(5000, 2850, 6500, M_COLOR_MATRIX_1,
                                M_COLOR_MATRIX_2),
            M_reference,
            decimal=7)

        np.testing.assert_almost_equal(
            interpolated_matrix(2850, 2850, 6500, M_COLOR_MATRIX_1,
                                M_COLOR_MATRIX_2),
            M_COLOR_MATRIX_1,
            decimal=7)

        np.testing.assert_almost_equal(
            interpolated_matrix(1000, 2850, 6500, M_COLOR_MATRIX_1,
                                M_COLOR_MATRIX_2),
            M_COLOR_MATRIX_1,
            decimal=7)

        np.testing.assert_almost_equal(
            interpolated_matrix(6500, 2850, 6500, M_COLOR_MATRIX_1,
                                M_COLOR_MATRIX_2),
            M_COLOR_MATRIX_2,
            decimal=7)

        np.testing.assert_almost_equal(
            interpolated_matrix(10000, 2850, 6500, M_COLOR_MATRIX_1,
                                M_COLOR_MATRIX_2),
            M_COLOR_MATRIX_2,
            decimal=7)


class TestXy_to_camera_neutral(unittest.TestCase):
    """
    Defines :func:`colour_hdri.models.adobe_dng.\
xy_to_camera_neutral` definition unit tests methods.
    """

    def test_xy_to_camera_neutral(self):
        """
        Tests :func:`colour_hdri.models.adobe_dng.\
xy_to_camera_neutral` definition.
        """

        np.testing.assert_almost_equal(
            xy_to_camera_neutral(
                np.array([0.32816244, 0.34698169]), 2850, 6500,
                M_COLOR_MATRIX_1, M_COLOR_MATRIX_2, np.identity(3),
                np.identity(3), ANALOG_BALANCE),
            np.array([0.41306999, 1.00000000, 0.64646500]),
            decimal=7)

        np.testing.assert_almost_equal(
            xy_to_camera_neutral(
                np.array([0.32816244, 0.34698169]), 2850,
                6500, M_COLOR_MATRIX_1, np.identity(3), np.identity(3),
                np.identity(3), ANALOG_BALANCE),
            np.array([0.42696744, 1.00000000, 0.63712786]),
            decimal=7)

        np.testing.assert_almost_equal(
            xy_to_camera_neutral(
                np.array([0.32816244, 0.34698169]), 2850, 6500,
                M_COLOR_MATRIX_1, M_COLOR_MATRIX_2, M_CAMERA_CALIBRATION_1,
                M_CAMERA_CALIBRATION_2, ANALOG_BALANCE),
            np.array([0.39667111, 1.00000000, 0.62474378]),
            decimal=7)


class TestCamera_neutral_to_xy(unittest.TestCase):
    """
    Defines :func:`colour_hdri.models.adobe_dng.\
camera_neutral_to_xy` definition unit tests methods.
    """

    def test_camera_neutral_to_xy(self):
        """
        Tests :func:`colour_hdri.models.adobe_dng.\
camera_neutral_to_xy` definition.
        """

        np.testing.assert_almost_equal(
            camera_neutral_to_xy(
                np.array([0.41306999, 1.00000000, 0.64646500]), 2850, 6500,
                M_COLOR_MATRIX_1, M_COLOR_MATRIX_2, np.identity(3),
                np.identity(3), ANALOG_BALANCE),
            np.array([0.32816244, 0.34698169]),
            decimal=7)

        np.testing.assert_almost_equal(
            camera_neutral_to_xy(
                np.array([0.42696744, 1.00000000, 0.63712786]), 2850,
                6500, M_COLOR_MATRIX_1, np.identity(3), np.identity(3),
                np.identity(3), ANALOG_BALANCE),
            np.array([0.32816244, 0.34698169]),
            decimal=7)

        np.testing.assert_almost_equal(
            camera_neutral_to_xy(
                np.array([0.39667111, 1.00000000, 0.62474378]), 2850, 6500,
                M_COLOR_MATRIX_1, M_COLOR_MATRIX_2, M_CAMERA_CALIBRATION_1,
                M_CAMERA_CALIBRATION_2, ANALOG_BALANCE),
            np.array([0.32816244, 0.34698169]),
            decimal=7)


class TestXYZ_to_camera_space_matrix(unittest.TestCase):
    """
    Defines :func:`colour_hdri.models.adobe_dng.\
XYZ_to_camera_space_matrix` definition unit tests methods.
    """

    def test_XYZ_to_camera_space_matrix(self):
        """
        Tests :func:`colour_hdri.models.adobe_dng.\
XYZ_to_camera_space_matrix` definition.
        """
        M_reference = np.array(
            [[0.47823951, 0.05098453, -0.07746894],
             [-0.76236708, 1.52266698, 0.25759538],
             [-0.14199759, 0.18561614, 0.66524555]])  # yapf: disable
        np.testing.assert_almost_equal(
            XYZ_to_camera_space_matrix(
                np.array([0.32816244, 0.34698169]), 2850, 6500,
                M_COLOR_MATRIX_1, M_COLOR_MATRIX_2, np.identity(3),
                np.identity(3), ANALOG_BALANCE),
            M_reference,
            decimal=7)

        M_reference = np.array(
            [[0.47823951, 0.05098453, -0.07746894],
             [-0.76236708, 1.52266698, 0.25759538],
             [-0.14199759, 0.18561614, 0.66524555]])  # yapf: disable
        np.testing.assert_almost_equal(
            XYZ_to_camera_space_matrix(
                np.array([0.32816244, 0.34698169]), 2850, 6500,
                M_COLOR_MATRIX_1, M_COLOR_MATRIX_2, np.identity(3),
                np.identity(3), ANALOG_BALANCE),
            M_reference,
            decimal=7)

        M_reference = np.array(
            [[0.47823951, 0.05098453, -0.07746894],
             [-0.76236708, 1.52266698, 0.25759538],
             [-0.14199759, 0.18561614, 0.66524555]])  # yapf: disable
        np.testing.assert_almost_equal(
            XYZ_to_camera_space_matrix(
                np.array([0.32816244, 0.34698169]), 2850, 6500,
                M_COLOR_MATRIX_1, M_COLOR_MATRIX_2, np.identity(3),
                np.identity(3), ANALOG_BALANCE),
            M_reference,
            decimal=7)


class TestCamera_space_to_XYZ_matrix(unittest.TestCase):
    """
    Defines :func:`colour_hdri.models.adobe_dng.\
camera_space_to_XYZ_matrix` definition unit tests methods.
    """

    def test_camera_space_to_XYZ_matrix(self):
        """
        Tests :func:`colour_hdri.models.adobe_dng.\
camera_space_to_XYZ_matrix` definition.
        """

        M_reference = np.array(
            [[2.16040871, -0.10410000, 0.27224985],
             [1.05333240, 0.66210000, -0.15035617],
             [0.12225531, -0.15620000, 1.43983046]])  # yapf: disable
        np.testing.assert_almost_equal(
            camera_space_to_XYZ_matrix(
                np.array([0.32816244, 0.34698169]), 2850, 6500,
                M_COLOR_MATRIX_1, M_COLOR_MATRIX_2, np.identity(3),
                np.identity(3), ANALOG_BALANCE, M_FORWARD_MATRIX_1,
                M_FORWARD_MATRIX_2),
            M_reference,
            decimal=7)

        M_reference = np.array(
            [[2.16040871, -0.10410000, 0.27224985],
             [1.05333240, 0.66210000, -0.15035617],
             [0.12225531, -0.15620000, 1.43983046]])  # yapf: disable
        np.testing.assert_almost_equal(
            camera_space_to_XYZ_matrix(
                np.array([0.32816244, 0.34698169]), 2850, 6500,
                M_COLOR_MATRIX_1, M_COLOR_MATRIX_2, np.identity(3),
                np.identity(3), ANALOG_BALANCE, M_FORWARD_MATRIX_1,
                M_FORWARD_MATRIX_2),
            M_reference,
            decimal=7)

        M_reference = np.array(
            [[2.16040871, -0.10410000, 0.27224985],
             [1.05333240, 0.66210000, -0.15035617],
             [0.12225531, -0.15620000, 1.43983046]])  # yapf: disable
        np.testing.assert_almost_equal(
            camera_space_to_XYZ_matrix(
                np.array([0.32816244, 0.34698169]), 2850, 6500,
                M_COLOR_MATRIX_1, M_COLOR_MATRIX_2, np.identity(3),
                np.identity(3), ANALOG_BALANCE, M_FORWARD_MATRIX_1,
                M_FORWARD_MATRIX_2),
            M_reference,
            decimal=7)

        M_reference = np.array(
            [[2.08092549, -0.08875351, 0.23804475],
             [1.00983630, 0.63301091, -0.14108066],
             [0.13472434, -0.17097714, 1.40232276]])  # yapf: disable
        np.testing.assert_almost_equal(
            camera_space_to_XYZ_matrix(
                np.array([0.32816244, 0.34698169]), 2850, 6500,
                M_COLOR_MATRIX_1, M_COLOR_MATRIX_2, np.identity(3),
                np.identity(3), ANALOG_BALANCE, np.identity(3),
                np.identity(3)),
            M_reference,
            decimal=7)


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
"""
Adobe DNG SDK Dataset
=====================

Defines various datasets objects for *Adobe DNG SDK*:

-   :attr:`colour_hdri.models.datasets.dng.CCS_ILLUMINANT_ADOBEDNG`
-   :attr:`colour_hdri.models.datasets.dng.\
CCT_ILLUMINANTS_ADOBEDNG`
-   :attr:`colour_hdri.models.datasets.dng.\
LIGHT_SOURCE_TAG_TO_DNG_ILLUMINANTS`

References
----------
-   :cite:`AdobeSystems2015c` : Adobe Systems. (2015). Adobe DNG SDK 1.4 -
    dng_sdk_1_4/dng_sdk/source/dng_camera_profile.cpp -
    dng_camera_profile::IlluminantToTemperature.
    http://download.adobe.com/pub/adobe/dng/dng_sdk_1_4.zip
-   :cite:`AdobeSystems2015d` : Adobe Systems. (2015). Adobe DNG SDK 1.4.
    http://download.adobe.com/pub/adobe/dng/dng_sdk_1_4.zip
-   :cite:`AdobeSystems2015e` : Adobe Systems. (2015). Adobe DNG SDK 1.4 -
    dng_sdk_1_4/dng_sdk/source/dng_tag_values.h - LightSource tag.
    http://download.adobe.com/pub/adobe/dng/dng_sdk_1_4.zip
"""

from __future__ import division, unicode_literals

from colour.colorimetry import CCS_ILLUMINANTS
from colour.utilities import CaseInsensitiveMapping

__author__ = 'Colour Developers'
__copyright__ = 'Copyright (C) 2015-2020 - Colour Developers'
__license__ = 'New BSD License - https://opensource.org/licenses/BSD-3-Clause'
__maintainer__ = 'Colour Developers'
__email__ = 'colour-developers@colour-science.org'
__status__ = 'Production'

__all__ = [
    'CCS_ILLUMINANT_ADOBEDNG', 'CCT_ILLUMINANTS_ADOBEDNG',
    'LIGHT_SOURCE_TAG_TO_DNG_ILLUMINANTS'
]

CCS_ILLUMINANT_ADOBEDNG = (
    CCS_ILLUMINANTS['CIE 1931 2 Degree Standard Observer']['D50'])
"""
*Adobe DNG SDK* default illuminant for *CIE XYZ* colourspace.

CCS_ILLUMINANT_ADOBEDNG : tuple
"""

CCT_ILLUMINANTS_ADOBEDNG = (CaseInsensitiveMapping({
    'Standard light A': 2850,
    'Tungsten (incandescent light)': 2850,
    'ISO studio tungsten': 3200,
    'D50': 5000,
    'D55': 5500,
    'Daylight': 5500,
    'Fine weather': 5500,
    'Flash': 5500,
    'Standard light B': 5500,
    'D65': 6500,
    'Standard light C': 6500,
    'Cloudy weather': 6500,
    'D75': 7500,
    'Shade': 7500,
    'Daylight fluorescent (D 5700 - 7100K)': (5700 + 7100) * 0.5,
    'Day white fluorescent (N 4600 - 5500K)': (4600 + 5400) * 0.5,
    'Cool white fluorescent (W 3800 - 4500K)': (3900 + 4500) * 0.5,
    'Fluorescent': (3900 + 4500) * 0.5,
    'White fluorescent (WW 3250 - 3800K)': (3200 + 3700) * 0.5,
    'Warm white fluorescent (L 2600 - 3250K)': (2600 + 3250) * 0.5
}))
"""
*Adobe DNG SDK* illuminants correlated colour temperature.

References
----------
:cite:`AdobeSystems2015c`

Notes
-----
-   The correlated colour temperature are given for the
    *CIE 1931 2 Degree Standard Observer*.

CCT_ILLUMINANTS_ADOBEDNG : CaseInsensitiveMapping
"""

LIGHT_SOURCE_TAG_TO_DNG_ILLUMINANTS = {
    1: 'Daylight',
    2: 'Fluorescent',
    3: 'Tungsten (incandescent light)',
    4: 'Flash',
    9: 'Fine weather',
    10: 'Cloudy weather',
    11: 'Shade',
    12: 'Daylight fluorescent (D 5700 - 7100K)',
    13: 'Day white fluorescent (N 4600 - 5500K)',
    14: 'Cool white fluorescent (W 3800 - 4500K)',
    15: 'White fluorescent (WW 3250 - 3800K)',
    16: 'Warm white fluorescent (L 2600 - 3250K)',
    17: 'Standard light A',
    18: 'Standard light B',
    19: 'Standard light C',
    20: 'D55',
    21: 'D65',
    22: 'D75',
    23: 'D50',
    24: 'ISO studio tungsten',
    255: 'Other'
}
"""
*Adobe DNG SDK* *LightSource Tag* indexes mapping to illuminants.

References
----------
:cite:`AdobeSystems2015e`

LIGHT_SOURCE_TAG_TO_DNG_ILLUMINANTS : CaseInsensitiveMapping
"""



# -*- coding: utf-8 -*-
"""
HDRI / Radiance Image Generation
================================

Defines HDRI / radiance image generation objects:

-   :func:`colour_hdri.image_stack_to_radiance_image`

See Also
--------
`Colour - HDRI - Examples Jupyter Notebooks
<https://github.com/colour-science/colour-hdri/\
blob/master/colour_hdri/examples>`__

References
----------
-   :cite:`Banterle2011n` : Banterle, F., Artusi, A., Debattista, K., &
    Chalmers, A. (2011). 2.1.1 Generating HDR Content by Combining Multiple
    Exposures. In Advanced High Dynamic Range Imaging. A K Peters/CRC Press.
    ISBN:978-1-56881-719-4
"""

from __future__ import division, unicode_literals

import numpy as np

from colour.utilities import tsplit, tstack, warning

from colour_hdri.exposure import average_luminance
from colour_hdri.generation import weighting_function_Debevec1997

__author__ = 'Colour Developers'
__copyright__ = 'Copyright (C) 2015-2020 - Colour Developers'
__license__ = 'New BSD License - https://opensource.org/licenses/BSD-3-Clause'
__maintainer__ = 'Colour Developers'
__email__ = 'colour-developers@colour-science.org'
__status__ = 'Production'

__all__ = ['image_stack_to_radiance_image']


def image_stack_to_radiance_image(
        image_stack,
        weighting_function=weighting_function_Debevec1997,
        weighting_average=False,
        camera_response_functions=None):
    """
    Generates a HDRI / radiance image from given image stack.

    Parameters
    ----------
    image_stack : colour_hdri.ImageStack
        Stack of single channel or multi-channel floating point images. The
        stack is assumed to be representing linear values except if
        ``camera_response_functions`` argument is provided.
    weighting_function : callable, optional
        Weighting function :math:`w`.
    weighting_average : bool, optional
         Enables weighting function :math:`w` computation on channels average
         instead of on a per channel basis.
    camera_response_functions : array_like, optional
        Camera response functions :math:`g(z)` of the imaging system / camera
        if the stack is representing non linear values.

    Returns
    -------
    ndarray
        Radiance image.

    Warnings
    --------
    If the image stack contains images with negative or equal to zero values,
    unpredictable results may occur and NaNs might be generated. It is
    thus recommended to encode the images in a wider RGB colourspace or clamp
    negative values.

    References
    ----------
    :cite:`Banterle2011n`
    """

    image_c = None
    weight_c = None
    for i, image in enumerate(image_stack):
        if image_c is None:
            image_c = np.zeros(image.data.shape)
            weight_c = np.zeros(image.data.shape)

        L = 1 / average_luminance(image.metadata.f_number,
                                  image.metadata.exposure_time,
                                  image.metadata.iso)

        if np.any(image.data <= 0):
            warning('"{0}" image channels contain negative or equal to zero '
                    'values, unpredictable results may occur! Please consider '
                    'encoding your images in a wider gamut RGB colourspace or '
                    'clamp negative values.'.format(image.path))

        if weighting_average and image.data.ndim == 3:
            average = np.average(image.data, axis=-1)

            weights = weighting_function(average)
            weights = np.rollaxis(weights[np.newaxis], 0, 3)
            if i == 0:
                weights[average >= 0.5] = 1
            if i == len(image_stack) - 1:
                weights[average <= 0.5] = 1
        else:
            weights = weighting_function(image.data)
            if i == 0:
                weights[image.data >= 0.5] = 1
            if i == len(image_stack) - 1:
                weights[image.data <= 0.5] = 1

        image_data = image.data
        if camera_response_functions is not None:
            samples = np.linspace(0, 1, camera_response_functions.shape[0])

            R, G, B = tsplit(image.data)
            R = np.interp(R, samples, camera_response_functions[..., 0])
            G = np.interp(G, samples, camera_response_functions[..., 1])
            B = np.interp(B, samples, camera_response_functions[..., 2])
            image_data = tstack([R, G, B])

        image_c += weights * image_data / L
        weight_c += weights

    if image_c is not None:
        image_c /= weight_c

    return image_c



# -*- coding: utf-8 -*-
"""
Tonemapping Operators Plotting
==============================

Defines the tonemapping operators plotting objects:

-   :func:`colour_hdri.plotting.plot_tonemapping_operator_image`
"""

from __future__ import division, unicode_literals

import matplotlib
import matplotlib.ticker
import numpy as np

from colour.plotting import (CONSTANTS_COLOUR_STYLE, artist, override_style,
                             render)

__author__ = 'Colour Developers'
__copyright__ = 'Copyright (C) 2015-2020 - Colour Developers'
__license__ = 'New BSD License - https://opensource.org/licenses/BSD-3-Clause'
__maintainer__ = 'Colour Developers'
__email__ = 'colour-developers@colour-science.org'
__status__ = 'Production'

__all__ = ['plot_tonemapping_operator_image']


@override_style()
def plot_tonemapping_operator_image(
        image,
        luminance_function,
        log_scale=False,
        cctf_encoding=CONSTANTS_COLOUR_STYLE.colour.colourspace.cctf_encoding,
        **kwargs):
    """
    Plots given tonemapped image with superimposed luminance mapping function.

    Parameters
    ----------
    image : array_like
         Tonemapped image to plot.
    luminance_function : callable
        Luminance mapping function.
    log_scale : bool, optional
        Use a log scale for plotting the luminance mapping function.
    cctf_encoding : callable, optional
        Encoding colour component transfer function / opto-electronic
        transfer function used for plotting.

    Other Parameters
    ----------------
    \\**kwargs : dict, optional
        {:func:`colour.plotting.render`},
        Please refer to the documentation of the previously listed definition.

    Returns
    -------
    tuple
        Current figure and axes.
    """

    settings = {'uniform': True}
    settings.update(kwargs)

    figure, axes = artist(**settings)

    shape = image.shape
    bounding_box = [0, 1, 0, 1]

    image = np.clip(cctf_encoding(image), 0, 1)
    axes.imshow(
        image,
        aspect=shape[0] / shape[1],
        extent=bounding_box,
        interpolation='nearest')

    axes.plot(
        np.linspace(0, 1, len(luminance_function)),
        luminance_function,
        color='red')

    settings = {
        'axes': axes,
        'bounding_box': bounding_box,
        'x_ticker': True,
        'y_ticker': True,
        'x_label': 'Input Luminance',
        'y_label': 'Output Luminance',
    }
    settings.update(kwargs)

    if log_scale:
        settings.update({
            'x_label': '$log_2$ Input Luminance',
            'x_ticker_locator': matplotlib.ticker.AutoMinorLocator(0.5)
        })
        matplotlib.pyplot.gca().set_xscale('log', basex=2)
        matplotlib.pyplot.gca().xaxis.set_major_formatter(
            matplotlib.ticker.ScalarFormatter())

    return render(**settings)



# !/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Defines unit tests for :mod:`colour_hdri.generation.weighting_functions`
module.
"""

from __future__ import division, unicode_literals

import numpy as np
import unittest

from colour_hdri.generation import (normal_distribution_function, hat_function,
                                    weighting_function_Debevec1997)

__author__ = 'Colour Developers'
__copyright__ = 'Copyright (C) 2015-2020 - Colour Developers'
__license__ = 'New BSD License - https://opensource.org/licenses/BSD-3-Clause'
__maintainer__ = 'Colour Developers'
__email__ = 'colour-developers@colour-science.org'
__status__ = 'Production'

__all__ = [
    'TestNormalDistributionFunction', 'TestHatFunction',
    'TestWeightingFunctionDebevec1997'
]


class TestNormalDistributionFunction(unittest.TestCase):
    """
    Defines :func:`colour_hdri.generation.weighting_functions.\
normal_distribution_function` definition unit tests methods.
    """

    def test_normal_distribution_function(self):
        """
        Tests :func:`colour_hdri.generation.weighting_functions.\
normal_distribution_function` definition.
        """

        np.testing.assert_almost_equal(
            normal_distribution_function(np.linspace(0, 1, 10)),
            np.array(
                [0.00386592, 0.03470859, 0.18002174, 0.53940751, 0.93371212,
                 0.93371212, 0.53940751, 0.18002174, 0.03470859, 0.00386592]),
            decimal=7)  # yapf: disable

        np.testing.assert_almost_equal(
            normal_distribution_function(np.linspace(0, 1, 10), 0, 1),
            np.array(
                [1.00000000, 0.99384617, 0.97561098, 0.94595947, 0.90595519,
                 0.85699689, 0.80073740, 0.73899130, 0.67363846, 0.60653066]),
            decimal=7)  # yapf: disable

        np.testing.assert_almost_equal(
            normal_distribution_function(np.linspace(0, 1, 10), 0.5, 0.5),
            np.array(
                [0.60653066, 0.73899130, 0.85699689, 0.94595947, 0.99384617,
                 0.99384617, 0.94595947, 0.85699689, 0.73899130, 0.60653066]),
            decimal=7)  # yapf: disable


class TestHatFunction(unittest.TestCase):
    """
    Defines :func:`colour_hdri.generation.weighting_functions.hat_function`
    definition unit tests methods.
    """

    def test_hat_function(self):
        """
        Tests :func:`colour_hdri.generation.weighting_functions.hat_function`
        definition.
        """

        np.testing.assert_almost_equal(
            hat_function(np.linspace(0, 1, 10)),
            np.array(
                [0.00000000, 0.95099207, 0.99913557, 0.99999812, 1.00000000,
                 1.00000000, 0.99999812, 0.99913557, 0.95099207, 0.00000000]),
            decimal=7)  # yapf: disable


class TestWeightingFunctionDebevec1997(unittest.TestCase):
    """
    Defines :func:`colour_hdri.generation.weighting_functions.\
weighting_function_Debevec1997` definition unit tests methods.
    """

    def test_weighting_function_Debevec1997(self):
        """
        Tests :func:`colour_hdri.generation.weighting_functions.\
weighting_function_Debevec1997` definition.
        """

        np.testing.assert_almost_equal(
            weighting_function_Debevec1997(np.linspace(0, 1, 10)),
            np.array(
                [0.00000000, 0.23273657, 0.48849105, 0.74424552, 1.00000000,
                 1.00000000, 0.74424552, 0.48849105, 0.23273657, 0.00000000]),
            decimal=7)  # yapf: disable

        np.testing.assert_almost_equal(
            weighting_function_Debevec1997(np.linspace(0, 1, 10), 0, 1),
            np.array(
                [0.00000000, 0.25000000, 0.50000000, 0.75000000, 1.00000000,
                 1.00000000, 0.75000000, 0.50000000, 0.25000000, 0.00000000]),
            decimal=7)  # yapf: disable

        np.testing.assert_almost_equal(
            weighting_function_Debevec1997(np.linspace(0, 1, 10), 0.25, 0.75),
            np.array(
                [0.00000000, 0.00000000, 0.00000000, 0.42857143, 1.00000000,
                 1.00000000, 0.42857143, 0.00000000, 0.00000000, 0.00000000]),
            decimal=7)  # yapf: disable


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-

from __future__ import absolute_import

from .dng import (
    RAW_CONVERTER, RAW_CONVERSION_ARGUMENTS, RAW_D_CONVERSION_ARGUMENTS,
    DNG_CONVERTER, DNG_CONVERSION_ARGUMENTS, DNG_EXIF_TAGS_BINDING,
    convert_raw_files_to_dng_files, convert_dng_files_to_intermediate_files,
    read_dng_files_exif_tags)

__all__ = [
    'RAW_CONVERTER', 'RAW_CONVERSION_ARGUMENTS', 'RAW_D_CONVERSION_ARGUMENTS',
    'DNG_CONVERTER', 'DNG_CONVERSION_ARGUMENTS', 'DNG_EXIF_TAGS_BINDING',
    'convert_raw_files_to_dng_files',
    'convert_dng_files_to_intermediate_files', 'read_dng_files_exif_tags'
]



# !/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Defines unit tests for :mod:`colour_hdri.utilities.exif` module.
"""

from __future__ import division, unicode_literals

import numpy as np
import os
import shutil
import tempfile
import unittest

from colour_hdri import TESTS_RESOURCES_DIRECTORY
from colour_hdri.utilities import filter_files, vivified_to_dict
from colour_hdri.utilities import (
    ExifTag, parse_exif_string, parse_exif_numeric, parse_exif_fraction,
    parse_exif_array, parse_exif_data, read_exif_tags, copy_exif_tags,
    update_exif_tags, delete_exif_tags, read_exif_tag, write_exif_tag)

__author__ = 'Colour Developers'
__copyright__ = 'Copyright (C) 2015-2020 - Colour Developers'
__license__ = 'New BSD License - https://opensource.org/licenses/BSD-3-Clause'
__maintainer__ = 'Colour Developers'
__email__ = 'colour-developers@colour-science.org'
__status__ = 'Production'

__all__ = [
    'FROBISHER_001_DIRECTORY', 'TestParseExifString', 'TestParseExifNumeric',
    'TestParseExifFraction', 'TestParseExifArray', 'TestParseExifData',
    'TestReadExifTags', 'TestCopyExifTags', 'TestUpdateExifTags',
    'TestDeleteExifTags', 'TestReadExifTag', 'TestWriteExifTag'
]

FROBISHER_001_DIRECTORY = os.path.join(TESTS_RESOURCES_DIRECTORY,
                                       'frobisher_001')


class TestParseExifString(unittest.TestCase):
    """
    Defines :func:`colour_hdri.utilities.exif.parse_exif_string` definition
    unit tests methods.
    """

    def test_parse_exif_string(self):
        """
        Tests :func:`colour_hdri.utilities.exif.parse_exif_string` definition.
        """

        exif_tag = ExifTag('EXIF', 'Make', 'Canon', '271')
        self.assertEqual(parse_exif_string(exif_tag), 'Canon')


class TestParseExifNumeric(unittest.TestCase):
    """
    Defines :func:`colour_hdri.utilities.exif.parse_exif_numeric` definition
    unit tests methods.
    """

    def test_parse_exif_numeric(self):
        """
        Tests :func:`colour_hdri.utilities.exif.parse_exif_numeric` definition.
        """

        exif_tag = ExifTag('EXIF', 'Focal Length', '16', '37386')
        self.assertEqual(parse_exif_numeric(exif_tag), 16)

        exif_tag = ExifTag('EXIF', 'Focal Length', '16', '37386')
        self.assertIsInstance(parse_exif_numeric(exif_tag, np.int_), np.int_)


class TestParseExifFraction(unittest.TestCase):
    """
    Defines :func:`colour_hdri.utilities.exif.parse_exif_fraction` definition
    unit tests methods.
    """

    def test_parse_exif_fraction(self):
        """
        Tests :func:`colour_hdri.utilities.exif.parse_exif_fraction`
        definition.
        """

        exif_tag = ExifTag('EXIF', 'Exposure Time', '0.01666666667', '33434')
        self.assertAlmostEqual(
            parse_exif_fraction(exif_tag), 0.01666666, places=7)

        exif_tag = ExifTag('EXIF', 'Exposure Time', '10/4000', '33434')
        self.assertAlmostEqual(
            parse_exif_fraction(exif_tag), 0.00250000, places=7)

        self.assertIsInstance(parse_exif_fraction(exif_tag, np.int_), np.int_)


class TestParseExifArray(unittest.TestCase):
    """
    Defines :func:`colour_hdri.utilities.exif.parse_exif_array` definition
    unit tests methods.
    """

    def test_parse_exif_array(self):
        """
        Tests :func:`colour_hdri.utilities.exif.parse_exif_array` definition.
        """

        exif_tag = ExifTag(
            'EXIF',
            'Color Matrix 1',
            ('0.5309 -0.0229 -0.0336 '
             '-0.6241 1.3265 0.3337 '
             '-0.0817 0.1215 0.6664'),
            '50721')  # yapf: disable
        np.testing.assert_array_equal(
            parse_exif_array(exif_tag),
            np.array([
                0.5309, -0.0229, -0.0336,
                -0.6241, 1.3265, 0.3337,
                -0.0817, 0.1215, 0.6664]))  # yapf: disable

        np.testing.assert_array_equal(
            parse_exif_array(exif_tag, shape=(3, 3)),
            np.array([
                [0.5309, -0.0229, -0.0336],
                [-0.6241, 1.3265, 0.3337],
                [-0.0817, 0.1215, 0.6664]]))  # yapf: disable


class TestParseExifData(unittest.TestCase):
    """
    Defines :func:`colour_hdri.utilities.exif.parse_exif_data` definition
    unit tests methods.
    """

    def test_parse_exif_data(self):
        """
        Tests :func:`colour_hdri.utilities.exif.parse_exif_data` definition.
        """

        self.assertListEqual(
            parse_exif_data(
                '[XMP]               - Description                     :'),
            ['XMP', '-', 'Description', ''])

        self.assertListEqual(
            parse_exif_data(
                '[EXIF]            296 Resolution Unit                 : 2'),
            ['EXIF', '296', 'Resolution Unit', '2'])

        self.assertListEqual(
            parse_exif_data(
                '[ICC_Profile]       8 Profile Version                 : 528'),
            ['ICC_Profile', '8', 'Profile Version', '528'])


class TestReadExifTags(unittest.TestCase):
    """
    Defines :func:`colour_hdri.utilities.exif.read_exif_tags` definition unit
    tests methods.
    """

    def test_read_exif_tags(self):
        """
        Tests :func:`colour_hdri.utilities.exif.read_exif_tags` definition.
        """

        test_jpg_image = filter_files(FROBISHER_001_DIRECTORY, ('jpg', ))[0]
        exif_data = vivified_to_dict(read_exif_tags(test_jpg_image))

        self.assertIsInstance(exif_data, type(dict()))

        self.assertListEqual(
            sorted(exif_data.keys()), [
                'Composite', 'EXIF', 'ExifTool', 'File', 'ICC_Profile', 'JFIF',
                'Photoshop', 'XMP'
            ])

        self.assertListEqual(
            sorted(exif_data['EXIF'].values()),
            [[ExifTag('EXIF', 'Camera Model Name', 'EOS 5D Mark II', '272')],
             [ExifTag('EXIF', 'Create Date', '2015:09:19 03:39:20', '36868')],
             [ExifTag('EXIF', 'Date/Time Original',
                      '2015:09:19 03:39:20', '36867')],
             [ExifTag('EXIF', 'Exif Image Height', '426', '40963')],
             [ExifTag('EXIF', 'Exif Image Width', '640', '40962')],
             [ExifTag('EXIF', 'Exposure Time', '0.125', '33434')],
             [ExifTag('EXIF', 'F Number', '8', '33437')],
             [ExifTag('EXIF', 'Focal Length', '16', '37386')],
             [ExifTag('EXIF', 'ISO', '100', '34855')],
             [ExifTag('EXIF', 'Make', 'Canon', '271')],
             [ExifTag('EXIF', 'Modify Date', '2015:09:19 03:39:20', '306')],
             [ExifTag('EXIF', 'Orientation', '1', '274')],
             [ExifTag('EXIF', 'Photometric Interpretation', '2', '262')],
             [ExifTag('EXIF', 'Resolution Unit', '2', '296')],
             [ExifTag('EXIF', 'Software', 'Photos 1.0.1', '305')],
             [ExifTag('EXIF', 'X Resolution', '72', '282')],
             [ExifTag('EXIF', 'Y Resolution', '72', '283')]])  # yapf: disable


class TestCopyExifTags(unittest.TestCase):
    """
    Defines :func:`colour_hdri.utilities.exif.copy_exif_tags` definition unit
    tests methods.
    """

    def setUp(self):
        """
        Initialises common tests attributes.
        """

        self._temporary_directory = tempfile.mkdtemp()

    def tearDown(self):
        """
        After tests actions.
        """

        shutil.rmtree(self._temporary_directory)

    def test_copy_exif_tags(self):
        """
        Tests :func:`colour_hdri.utilities.exif.copy_exif_tags` definition.
        """

        reference_jpg_image = filter_files(FROBISHER_001_DIRECTORY,
                                           ('jpg', ))[0]
        test_jpg_image = os.path.join(self._temporary_directory,
                                      os.path.basename(reference_jpg_image))

        shutil.copyfile(reference_jpg_image, test_jpg_image)
        self.assertEqual(read_exif_tag(test_jpg_image, 'Aperture'), '8.0')
        delete_exif_tags(test_jpg_image)
        self.assertEqual(read_exif_tag(test_jpg_image, 'Aperture'), '')
        copy_exif_tags(reference_jpg_image, test_jpg_image)
        self.assertEqual(read_exif_tag(test_jpg_image, 'Aperture'), '8.0')


class TestUpdateExifTags(unittest.TestCase):
    """
    Defines :func:`colour_hdri.utilities.exif.update_exif_tags` definition unit
    tests methods.
    """

    def setUp(self):
        """
        Initialises common tests attributes.
        """

        self._temporary_directory = tempfile.mkdtemp()

    def tearDown(self):
        """
        After tests actions.
        """

        shutil.rmtree(self._temporary_directory)

    def test_update_exif_tags(self):
        """
        Tests :func:`colour_hdri.utilities.exif.update_exif_tags` definition.
        """

        reference_jpg_images = filter_files(FROBISHER_001_DIRECTORY, ('jpg', ))
        test_jpg_images = []
        for reference_jpg_image in reference_jpg_images:
            test_jpg_image = os.path.join(
                self._temporary_directory,
                os.path.basename(reference_jpg_image))
            shutil.copyfile(reference_jpg_image, test_jpg_image)
            delete_exif_tags(test_jpg_image)
            self.assertEqual(read_exif_tag(test_jpg_image, 'Aperture'), '')
            test_jpg_images.append(test_jpg_image)

        update_exif_tags(zip(reference_jpg_images, test_jpg_images))
        for test_jpg_image in test_jpg_images:
            self.assertEqual(read_exif_tag(test_jpg_image, 'Aperture'), '8.0')


class TestDeleteExifTags(unittest.TestCase):
    """
    Defines :func:`colour_hdri.utilities.exif.delete_exif_tags` definition unit
    tests methods.
    """

    def setUp(self):
        """
        Initialises common tests attributes.
        """

        self._temporary_directory = tempfile.mkdtemp()

    def tearDown(self):
        """
        After tests actions.
        """

        shutil.rmtree(self._temporary_directory)

    def test_delete_exif_tags(self):
        """
        Tests :func:`colour_hdri.utilities.exif.delete_exif_tags` definition.
        """

        reference_jpg_image = filter_files(FROBISHER_001_DIRECTORY,
                                           ('jpg', ))[0]
        test_jpg_image = os.path.join(self._temporary_directory,
                                      os.path.basename(reference_jpg_image))

        shutil.copyfile(reference_jpg_image, test_jpg_image)
        self.assertEqual(read_exif_tag(test_jpg_image, 'Aperture'), '8.0')
        delete_exif_tags(test_jpg_image)
        self.assertEqual(read_exif_tag(test_jpg_image, 'Aperture'), '')


class TestReadExifTag(unittest.TestCase):
    """
    Defines :func:`colour_hdri.utilities.exif.read_exif_tag` definition unit
    tests methods.
    """

    def test_read_exif_tag(self):
        """
        Tests :func:`colour_hdri.utilities.exif.read_exif_tag` definition.
        """

        test_jpg_image = filter_files(FROBISHER_001_DIRECTORY, ('jpg', ))[0]

        self.assertEqual(read_exif_tag(test_jpg_image, 'Aperture'), '8.0')
        self.assertEqual(read_exif_tag(test_jpg_image, 'ExposureTime'), '1/8')
        self.assertEqual(read_exif_tag(test_jpg_image, 'ISO'), '100')


class TestWriteExifTag(unittest.TestCase):
    """
    Defines :func:`colour_hdri.utilities.exif.write_exif_tag` definition unit
    tests methods.
    """

    def setUp(self):
        """
        Initialises common tests attributes.
        """

        self._temporary_directory = tempfile.mkdtemp()

    def tearDown(self):
        """
        After tests actions.
        """

        shutil.rmtree(self._temporary_directory)

    def test_write_exif_tag(self):
        """
        Tests :func:`colour_hdri.utilities.exif.write_exif_tag` definition.
        """

        reference_jpg_image = filter_files(FROBISHER_001_DIRECTORY,
                                           ('jpg', ))[0]
        test_jpg_image = os.path.join(self._temporary_directory,
                                      os.path.basename(reference_jpg_image))

        shutil.copyfile(reference_jpg_image, test_jpg_image)
        # *Aperture* exif tag is not writeable, changing for *FNumber*.
        self.assertEqual(read_exif_tag(test_jpg_image, 'FNumber'), '8.0')
        write_exif_tag(test_jpg_image, 'FNumber', '16.0')
        self.assertEqual(read_exif_tag(test_jpg_image, 'FNumber'), '16.0')


if __name__ == '__main__':
    unittest.main()



# !/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Defines unit tests for :mod:`colour_hdri.calibration.absolute_luminance`
module.
"""

from __future__ import division, unicode_literals

import numpy as np
import os
import unittest

from colour import read_image

from colour_hdri import TESTS_RESOURCES_DIRECTORY
from colour_hdri.calibration import (
    upper_hemisphere_illuminance_weights_Lagarde2016,
    absolute_luminance_calibration_Lagarde2016)
from colour_hdri.calibration.absolute_luminance import (
    upper_hemisphere_illuminance_Lagarde2016)

__author__ = 'Colour Developers'
__copyright__ = 'Copyright (C) 2015-2020 - Colour Developers'
__license__ = 'New BSD License - https://opensource.org/licenses/BSD-3-Clause'
__maintainer__ = 'Colour Developers'
__email__ = 'colour-developers@colour-science.org'
__status__ = 'Production'

__all__ = [
    'TestUpperHemisphereIlluminanceLagarde2016',
    'TestUpperHemisphereIlluminanceWeightsLagarde2016',
    'TestAbsoluteLuminanceCalibrationLagarde2016'
]

UNITY_001_DIRECTORY = os.path.join(TESTS_RESOURCES_DIRECTORY, 'unity_001')

CALIBRATION_DIRECTORY = os.path.join(TESTS_RESOURCES_DIRECTORY, 'colour_hdri',
                                     'calibration')


class TestUpperHemisphereIlluminanceLagarde2016(unittest.TestCase):
    """
    Defines :func:`colour_hdri.calibration.absolute_luminance.\
upper_hemisphere_illuminance_Lagarde2016` definition unit tests methods.
    """

    def test_upper_hemisphere_illuminance_Lagarde2016(self):
        """
        Tests :func:`colour_hdri.calibration.absolute_luminance.\
upper_hemisphere_illuminance_Lagarde2016` definition.
        """

        self.assertAlmostEqual(
            upper_hemisphere_illuminance_Lagarde2016(np.ones((16, 32, 3))),
            2.934469165342606,
            places=7)

        self.assertAlmostEqual(
            upper_hemisphere_illuminance_Lagarde2016(
                np.ones((16, 32, 3)) * 10),
            29.344691653426061,
            places=7)

        self.assertAlmostEqual(
            upper_hemisphere_illuminance_Lagarde2016(
                np.ones((16, 32, 3)) * 0.1),
            0.293446916534261,
            places=7)


class TestUpperHemisphereIlluminanceWeightsLagarde2016(unittest.TestCase):
    """
    Defines :func:`colour_hdri.calibration.absolute_luminance.\
upper_hemisphere_illuminance_weights_Lagarde2016` definition unit tests
    methods.
    """

    def test_upper_hemisphere_illuminance_weights_Lagarde2016(self):
        """
        Tests :func:`colour_hdri.calibration.absolute_luminance.\
upper_hemisphere_illuminance_weights_Lagarde2016` definition.
        """

        weights = np.array(
            [[0.00000000],
             [1.98673676],
             [3.89213628],
             [5.63819129],
             [7.15341808],
             [8.37578310],
             [9.25524257],
             [9.75579132],
             [9.85693683],
             [9.55453819],
             [8.86097564],
             [7.80464369],
             [6.42878872],
             [4.78973839],
             [2.95459560],
             [0.99849132],
             [0.00000000],
             [0.00000000],
             [0.00000000],
             [0.00000000],
             [0.00000000],
             [0.00000000],
             [0.00000000],
             [0.00000000],
             [0.00000000],
             [0.00000000],
             [0.00000000],
             [0.00000000],
             [0.00000000],
             [0.00000000],
             [0.00000000],
             [0.00000000]])  # yapf: disable

        np.testing.assert_almost_equal(
            upper_hemisphere_illuminance_weights_Lagarde2016(32, 16),
            np.tile(weights, (1, 16)),
            decimal=7)


class TestAbsoluteLuminanceCalibrationLagarde2016(unittest.TestCase):
    """
    Defines :func:`colour_hdri.calibration.absolute_luminance.\
absolute_luminance_calibration_Lagarde2016` definition unit tests methods.
    """

    def test_absolute_luminance_calibration_Lagarde2016(self):
        """
        Tests :func:`colour_hdri.calibration.absolute_luminance.\
absolute_luminance_calibration_Lagarde2016` definition.
        """

        # Unity Technologies. (2016). Treasure Island - white balanced.exr.
        # Retrieved August 30, 2016, from http://blog.selfshadow.com/\
        # publications/s2016-shading-course/unity/supplemental/\
        # Treasure Island - white balanced.exr

        reference_exr_file = read_image(
            str(
                os.path.join(UNITY_001_DIRECTORY,
                             'Unity_Treasure_Island_White_Balanced.exr')))

        test_exr_file = read_image(
            str(
                os.path.join(
                    CALIBRATION_DIRECTORY,
                    'Unity_Treasure_Island_White_Balanced_Absolute.exr')))

        np.testing.assert_allclose(
            absolute_luminance_calibration_Lagarde2016(reference_exr_file,
                                                       51000),
            test_exr_file,
            rtol=0.0000001,
            atol=0.0000001)


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['colour_hdri',
 'colour_hdri.calibration',
 'colour_hdri.calibration.tests',
 'colour_hdri.exposure',
 'colour_hdri.exposure.tests',
 'colour_hdri.generation',
 'colour_hdri.generation.tests',
 'colour_hdri.models',
 'colour_hdri.models.datasets',
 'colour_hdri.models.tests',
 'colour_hdri.plotting',
 'colour_hdri.process',
 'colour_hdri.process.tests',
 'colour_hdri.recovery',
 'colour_hdri.recovery.tests',
 'colour_hdri.sampling',
 'colour_hdri.sampling.tests',
 'colour_hdri.tonemapping',
 'colour_hdri.tonemapping.global_operators',
 'colour_hdri.tonemapping.global_operators.tests',
 'colour_hdri.utilities',
 'colour_hdri.utilities.tests']

package_data = \
{'': ['*'],
 'colour_hdri': ['examples/examples_absolute_luminance_calibration_and_photometric_exposure_conversion.ipynb',
                 'examples/examples_absolute_luminance_calibration_and_photometric_exposure_conversion.ipynb',
                 'examples/examples_absolute_luminance_calibration_and_photometric_exposure_conversion.ipynb',
                 'examples/examples_absolute_luminance_calibration_and_photometric_exposure_conversion.ipynb',
                 'examples/examples_absolute_luminance_calibration_and_photometric_exposure_conversion.ipynb',
                 'examples/examples_absolute_luminance_calibration_and_photometric_exposure_conversion.ipynb',
                 'examples/examples_absolute_luminance_calibration_and_photometric_exposure_conversion.ipynb',
                 'examples/examples_absolute_luminance_calibration_and_photometric_exposure_conversion.ipynb',
                 'examples/examples_adobe_dng_sdk_colour_processing.ipynb',
                 'examples/examples_adobe_dng_sdk_colour_processing.ipynb',
                 'examples/examples_adobe_dng_sdk_colour_processing.ipynb',
                 'examples/examples_adobe_dng_sdk_colour_processing.ipynb',
                 'examples/examples_adobe_dng_sdk_colour_processing.ipynb',
                 'examples/examples_adobe_dng_sdk_colour_processing.ipynb',
                 'examples/examples_adobe_dng_sdk_colour_processing.ipynb',
                 'examples/examples_adobe_dng_sdk_colour_processing.ipynb',
                 'examples/examples_global_tonemapping_operators.ipynb',
                 'examples/examples_global_tonemapping_operators.ipynb',
                 'examples/examples_global_tonemapping_operators.ipynb',
                 'examples/examples_global_tonemapping_operators.ipynb',
                 'examples/examples_global_tonemapping_operators.ipynb',
                 'examples/examples_global_tonemapping_operators.ipynb',
                 'examples/examples_global_tonemapping_operators.ipynb',
                 'examples/examples_global_tonemapping_operators.ipynb',
                 'examples/examples_merge_from_ldr_files.ipynb',
                 'examples/examples_merge_from_ldr_files.ipynb',
                 'examples/examples_merge_from_ldr_files.ipynb',
                 'examples/examples_merge_from_ldr_files.ipynb',
                 'examples/examples_merge_from_ldr_files.ipynb',
                 'examples/examples_merge_from_ldr_files.ipynb',
                 'examples/examples_merge_from_ldr_files.ipynb',
                 'examples/examples_merge_from_ldr_files.ipynb',
                 'examples/examples_merge_from_raw_files.ipynb',
                 'examples/examples_merge_from_raw_files.ipynb',
                 'examples/examples_merge_from_raw_files.ipynb',
                 'examples/examples_merge_from_raw_files.ipynb',
                 'examples/examples_merge_from_raw_files.ipynb',
                 'examples/examples_merge_from_raw_files.ipynb',
                 'examples/examples_merge_from_raw_files.ipynb',
                 'examples/examples_merge_from_raw_files.ipynb',
                 'examples/examples_merge_from_raw_files_using_rawpy.ipynb',
                 'examples/examples_merge_from_raw_files_using_rawpy.ipynb',
                 'examples/examples_merge_from_raw_files_using_rawpy.ipynb',
                 'examples/examples_merge_from_raw_files_using_rawpy.ipynb',
                 'examples/examples_merge_from_raw_files_using_rawpy.ipynb',
                 'examples/examples_merge_from_raw_files_using_rawpy.ipynb',
                 'examples/examples_merge_from_raw_files_using_rawpy.ipynb',
                 'examples/examples_merge_from_raw_files_using_rawpy.ipynb',
                 'examples/examples_merge_from_raw_files_with_post_demosaicing.ipynb',
                 'examples/examples_merge_from_raw_files_with_post_demosaicing.ipynb',
                 'examples/examples_merge_from_raw_files_with_post_demosaicing.ipynb',
                 'examples/examples_merge_from_raw_files_with_post_demosaicing.ipynb',
                 'examples/examples_merge_from_raw_files_with_post_demosaicing.ipynb',
                 'examples/examples_merge_from_raw_files_with_post_demosaicing.ipynb',
                 'examples/examples_merge_from_raw_files_with_post_demosaicing.ipynb',
                 'examples/examples_merge_from_raw_files_with_post_demosaicing.ipynb',
                 'examples/examples_variance_minimization_light_probe_sampling.ipynb',
                 'examples/examples_variance_minimization_light_probe_sampling.ipynb',
                 'examples/examples_variance_minimization_light_probe_sampling.ipynb',
                 'examples/examples_variance_minimization_light_probe_sampling.ipynb',
                 'examples/examples_variance_minimization_light_probe_sampling.ipynb',
                 'examples/examples_variance_minimization_light_probe_sampling.ipynb',
                 'examples/examples_variance_minimization_light_probe_sampling.ipynb',
                 'examples/examples_variance_minimization_light_probe_sampling.ipynb',
                 'resources/colour-hdri-examples-datasets/*',
                 'resources/colour-hdri-tests-datasets/*']}

install_requires = \
['colour-science>=0.3.16,<0.4.0', 'recordclass']

extras_require = \
{'development': ['biblib-simple',
                 'coverage',
                 'coveralls',
                 'flake8',
                 'invoke',
                 'jupyter',
                 'mock',
                 'nbformat>=4,<5',
                 'nose',
                 'pre-commit',
                 'pytest',
                 'restructuredtext-lint',
                 'sphinx<=3.1.2',
                 'sphinx_rtd_theme',
                 'sphinxcontrib-bibtex',
                 'toml',
                 'twine',
                 'yapf==0.23'],
 'optional': ['colour-demosaicing', 'rawpy'],
 'plotting': ['backports.functools_lru_cache', 'matplotlib'],
 'read-the-docs': ['mock', 'numpy', 'sphinxcontrib-bibtex']}

setup_kwargs = {
    'name': 'colour-hdri',
    'version': '0.1.8',
    'description': 'HDRI / Radiance image processing algorithms for Python',
    'long_description': "Colour - HDRI\n=============\n\n.. start-badges\n\n|actions| |coveralls| |codacy| |version|\n\n.. |actions| image:: https://img.shields.io/github/workflow/status/colour-science/colour-hdri/Continuous%20Integration?label=actions&logo=github&style=flat-square\n    :target: https://github.com/colour-science/colour-hdri/actions\n    :alt: Develop Build Status\n.. |coveralls| image:: http://img.shields.io/coveralls/colour-science/colour-hdri/develop.svg?style=flat-square\n    :target: https://coveralls.io/r/colour-science/colour-hdri\n    :alt: Coverage Status\n.. |codacy| image:: https://img.shields.io/codacy/grade/290ad2c23b0749b99a1d548ca47d9062/develop.svg?style=flat-square\n    :target: https://www.codacy.com/app/colour-science/colour-hdri\n    :alt: Code Grade\n.. |version| image:: https://img.shields.io/pypi/v/colour-hdri.svg?style=flat-square\n    :target: https://pypi.org/project/colour-hdri\n    :alt: Package Version\n\n.. end-badges\n\nA `Python <https://www.python.org/>`__ package implementing various\nHDRI / Radiance image processing algorithms.\n\nIt is open source and freely available under the\n`New BSD License <https://opensource.org/licenses/BSD-3-Clause>`__ terms.\n\n..  image:: https://raw.githubusercontent.com/colour-science/colour-hdri/master/docs/_static/Radiance_001.png\n\n.. contents:: **Table of Contents**\n    :backlinks: none\n    :depth: 3\n\n.. sectnum::\n\nFeatures\n--------\n\nThe following features are available:\n\n-   HDRI / Radiance Image Generation\n-   Debevec (1997) Camera Response Function Computation\n-   Grossberg (2003) Histogram Based Image Sampling\n-   Variance Minimization Light Probe Sampling\n-   Global Tonemapping Operators\n-   Adobe DNG SDK Colour Processing\n-   Absolute Luminance Calibration\n-   Digital Still Camera (DSC) Exposure Model\n-   Raw Processing Helpers\n\nInstallation\n------------\n\nBecause of their size, the resources dependencies needed to run the various\nexamples and unit tests are not provided within the Pypi package. They are\nseparately available as\n`Git Submodules <https://git-scm.com/book/en/v2/Git-Tools-Submodules>`__\nwhen cloning the\n`repository <https://github.com/colour-science/colour-hdri>`__.\n\nPrimary Dependencies\n^^^^^^^^^^^^^^^^^^^^\n\n**Colour - HDRI** requires various dependencies in order to run:\n\n-   `python>=2.7 <https://www.python.org/download/releases/>`__ or\n    `python>=3.5 <https://www.python.org/download/releases/>`__\n-   `colour-science <https://pypi.org/project/colour-science/>`__\n-   `recordclass <https://pypi.org/project/recordclass/>`__\n\nOptional Features Dependencies\n^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^\n\n-   `colour-demosaicing <https://pypi.org/project/colour-demosaicing/>`__\n-   `Adobe DNG Converter <https://www.adobe.com/support/downloads/product.jsp?product=106&platform=Mac>`__\n-   `dcraw <https://www.cybercom.net/~dcoffin/dcraw/>`__\n-   `ExifTool <http://www.sno.phy.queensu.ca/~phil/exiftool/>`__\n-   `rawpy <https://pypi.org/project/rawpy/>`__\n\nPypi\n^^^^\n\nOnce the dependencies are satisfied, **Colour - HDRI** can be installed from\nthe `Python Package Index <http://pypi.python.org/pypi/colour-hdri>`__ by\nissuing this command in a shell::\n\n\tpip install --user colour-hdri\n\nThe optional features dependencies are installed as follows::\n\n    pip install --user 'colour-hdri[optional]'\n\nThe figures plotting dependencies are installed as follows::\n\n    pip install --user 'colour-hdri[plotting]'\n\nThe overall development dependencies are installed as follows::\n\n    pip install --user 'colour-hdri[development]'\n\nUsage\n-----\n\nAPI\n^^^\n\nThe main reference for `Colour - HDRI <https://github.com/colour-science/colour-hdri>`__\nis the `Colour - HDRI Manual <https://colour-hdri.readthedocs.io/en/latest/manual.html>`__.\n\nExamples\n^^^^^^^^\n\nVarious usage examples are available from the\n`examples directory <https://github.com/colour-science/colour-hdri/tree/master/colour_hdri/examples>`__.\n\nContributing\n------------\n\nIf you would like to contribute to `Colour - HDRI <https://github.com/colour-science/colour-hdri>`__,\nplease refer to the following `Contributing <https://www.colour-science.org/contributing/>`__\nguide for `Colour <https://github.com/colour-science/colour>`__.\n\nBibliography\n------------\n\nThe bibliography is available in the repository in\n`BibTeX <https://github.com/colour-science/colour-hdri/blob/develop/BIBLIOGRAPHY.bib>`__\nformat.\n\nCode of Conduct\n---------------\n\nThe *Code of Conduct*, adapted from the `Contributor Covenant 1.4 <https://www.contributor-covenant.org/version/1/4/code-of-conduct.html>`__,\nis available on the `Code of Conduct <https://www.colour-science.org/code-of-conduct/>`__ page.\n\nAbout\n-----\n\n| **Colour - HDRI** by Colour Developers\n| Copyright © 2015-2020 – Colour Developers – `colour-developers@colour-science.org <colour-developers@colour-science.org>`__\n| This software is released under terms of New BSD License: https://opensource.org/licenses/BSD-3-Clause\n| `https://github.com/colour-science/colour-hdri <https://github.com/colour-science/colour-hdri>`__\n",
    'author': 'Colour Developers',
    'author_email': 'colour-developers@colour-science.org',
    'maintainer': 'Colour Developers',
    'maintainer_email': 'colour-developers@colour-science.org',
    'url': 'https://www.colour-science.org/',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'python_requires': '>=2.7, !=3.0.*, !=3.1.*, !=3.2.*, !=3.3.*, !=3.4.*, !=3.5.*',
}


setup(**setup_kwargs)



# -*- coding: utf-8 -*-

from __future__ import absolute_import

from .highlights import highlights_recovery_blend, highlights_recovery_LCHab

__all__ = ['highlights_recovery_blend', 'highlights_recovery_LCHab']



# -*- coding: utf-8 -*-

from __future__ import absolute_import

from .weighting_functions import (normal_distribution_function, hat_function,
                                  weighting_function_Debevec1997)
from .radiance import image_stack_to_radiance_image

__all__ = []
__all__ += [
    'normal_distribution_function', 'hat_function',
    'weighting_function_Debevec1997'
]
__all__ += ['image_stack_to_radiance_image']



# -*- coding: utf-8 -*-

from __future__ import absolute_import

from .radiance import plot_radiance_image_strip
from .tonemapping import plot_tonemapping_operator_image

__all__ = []
__all__ += ['plot_radiance_image_strip']
__all__ += ['plot_tonemapping_operator_image']



# !/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Defines unit tests for :mod:`colour_hdri.process.adobe_dng` module.
"""

from __future__ import division, unicode_literals

import numpy as np
import os
import platform
import shutil
import tempfile
import unittest
import zipfile

from colour import read_image

from colour_hdri import TESTS_RESOURCES_DIRECTORY
from colour_hdri.process import (convert_raw_files_to_dng_files,
                                 convert_dng_files_to_intermediate_files,
                                 read_dng_files_exif_tags)
from colour_hdri.utilities import filter_files

__author__ = 'Colour Developers'
__copyright__ = 'Copyright (C) 2015-2020 - Colour Developers'
__license__ = 'New BSD License - https://opensource.org/licenses/BSD-3-Clause'
__maintainer__ = 'Colour Developers'
__email__ = 'colour-developers@colour-science.org'
__status__ = 'Production'

__all__ = [
    'FROBISHER_001_DIRECTORY', 'PROCESS_DIRECTORY', 'RAW_IMAGES',
    'TestConvertRawFilesToDngFiles', 'TestConvertDngFilesToIntermediateFiles'
]

FROBISHER_001_DIRECTORY = os.path.join(TESTS_RESOURCES_DIRECTORY,
                                       'frobisher_001')

PROCESS_DIRECTORY = os.path.join(TESTS_RESOURCES_DIRECTORY, 'colour_hdri',
                                 'process')

RAW_IMAGES = filter_files(FROBISHER_001_DIRECTORY, ('CR2', ))


class TestConvertRawFilesToDngFiles(unittest.TestCase):
    """
    Defines :func:`colour_hdri.process.adobe_dng.\
convert_raw_files_to_dng_files` definition unit tests methods.
    """

    def setUp(self):
        """
        Initialises common tests attributes.
        """

        self._temporary_directory = tempfile.mkdtemp()

    def tearDown(self):
        """
        After tests actions.
        """

        shutil.rmtree(self._temporary_directory)

    def test_convert_raw_files_to_dng_files(self):
        """
        Tests :func:`colour_hdri.process.adobe_dng.\
convert_raw_files_to_dng_files` definition.
        """

        if platform.system() not in ('Windows', 'Microsoft'):
            # *Adobe DNG Converter* is not available on *Linux*, and is not
            # reproducible on *macOS* thus we skip this unit test.
            return

        reference_dng_files = sorted(
            filter_files(PROCESS_DIRECTORY, ('dng', )))
        test_dng_files = sorted(
            convert_raw_files_to_dng_files(RAW_IMAGES,
                                           self._temporary_directory))

        for test_dng_file, reference_dng_file in zip(test_dng_files,
                                                     reference_dng_files):
            np.testing.assert_almost_equal(
                read_image(str(test_dng_file)),
                read_image(str(reference_dng_file)),
                decimal=7)


class TestConvertDngFilesToIntermediateFiles(unittest.TestCase):
    """
    Defines :func:`colour_hdri.process.adobe_dng.\
convert_dng_files_to_intermediate_files` definition unit tests methods.
    """

    def setUp(self):
        """
        Initialises common tests attributes.
        """

        self._temporary_directory = tempfile.mkdtemp()

    def tearDown(self):
        """
        After tests actions.
        """

        shutil.rmtree(self._temporary_directory)

    def test_convert_dng_files_to_intermediate_files(self):
        """
        Tests :func:`colour_hdri.process.adobe_dng.\
convert_dng_files_to_intermediate_files` definition.
        """

        reference_dng_files = sorted(
            filter_files(PROCESS_DIRECTORY, ('dng', )))
        tests_dng_files = [
            os.path.join(self._temporary_directory,
                         os.path.basename(reference_dng_file))
            for reference_dng_file in reference_dng_files
        ]
        for reference_dng_file, tests_dng_file in zip(reference_dng_files,
                                                      tests_dng_files):
            shutil.copyfile(reference_dng_file, tests_dng_file)

        reference_zip_files = sorted(
            filter_files(PROCESS_DIRECTORY, ('zip', )))

        for reference_zip_file in reference_zip_files:
            with zipfile.ZipFile(reference_zip_file) as zip_file:
                tiff_file_name = os.path.basename(reference_zip_file).replace(
                    '.zip', '')
                with open(
                        os.path.join(self._temporary_directory,
                                     tiff_file_name),
                        'wb') as reference_tiff_file:
                    reference_tiff_file.write(zip_file.read(tiff_file_name))

        reference_tiff_files = sorted(
            filter_files(self._temporary_directory, ('tiff', )))

        # for reference_tiff_file in reference_tiff_files:
        #     os.chdir(os.path.dirname(reference_tiff_file))
        #     with zipfile.ZipFile(
        #             '{0}.zip'.format(reference_tiff_file),
        #             mode='w') as zip_file:
        #         zip_file.write(
        #             os.path.basename(reference_tiff_file),
        #             compress_type=zipfile.ZIP_DEFLATED)
        #         os.remove(reference_tiff_file)

        test_tiff_files = sorted(
            convert_dng_files_to_intermediate_files(tests_dng_files,
                                                    self._temporary_directory))

        for test_tiff_file, reference_tiff_file in zip(test_tiff_files,
                                                       reference_tiff_files):
            np.testing.assert_almost_equal(
                read_image(str(test_tiff_file)),
                read_image(str(reference_tiff_file)),
                decimal=7)


class TestReadDngFilesExifTags(unittest.TestCase):
    """
    Defines :func:`colour_hdri.process.adobe_dng.\
read_dng_files_exif_tags` definition unit tests methods.
    """

    def test_read_dng_files_exif_tags(self):
        """
        Tests :func:`colour_hdri.process.adobe_dng.\
read_dng_files_exif_tags` definition.
        """

        reference_dng_files = sorted(
            filter_files(PROCESS_DIRECTORY, ('dng', )))
        exif_tags = read_dng_files_exif_tags(reference_dng_files)
        self.assertEqual(len(exif_tags), 3)
        self.assertIn('EXIF', exif_tags[0])
        self.assertIn('Make', exif_tags[0]['EXIF'])

        self.assertAlmostEquals(
            exif_tags[0]['EXIF']['Exposure Time'], 0.12500000, places=7)

        np.testing.assert_array_equal(
            exif_tags[0]['EXIF']['Reduction Matrix 1'], np.identity(3))


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-

from __future__ import absolute_import

from .dng import (CCS_ILLUMINANT_ADOBEDNG, CCT_ILLUMINANTS_ADOBEDNG,
                  LIGHT_SOURCE_TAG_TO_DNG_ILLUMINANTS)

__all__ = [
    'CCS_ILLUMINANT_ADOBEDNG', 'CCT_ILLUMINANTS_ADOBEDNG',
    'LIGHT_SOURCE_TAG_TO_DNG_ILLUMINANTS'
]



# -*- coding: utf-8 -*-

from __future__ import absolute_import

from .absolute_luminance import (
    upper_hemisphere_illuminance_weights_Lagarde2016,
    absolute_luminance_calibration_Lagarde2016)
from .debevec1997 import g_solve, camera_response_functions_Debevec1997

__all__ = [
    'upper_hemisphere_illuminance_weights_Lagarde2016',
    'absolute_luminance_calibration_Lagarde2016'
]
__all__ += ['g_solve', 'camera_response_functions_Debevec1997']
